@@ -29,12 +29,17 @@
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
 
-#include <libempathy/empathy-account-manager.h>
+#include <telepathy-glib/account-manager.h>
+#include <telepathy-glib/util.h>
+
 #include <libempathy/empathy-utils.h>
 
 #include "empathy-ui-utils.h"
 #include "empathy-account-chooser.h"
 
+#define DEBUG_FLAG EMPATHY_DEBUG_OTHER
+#include <libempathy/empathy-debug.h>
+
 /**
  * SECTION:empathy-account-chooser
  * @title:EmpathyAccountChooser
@@ -54,24 +59,36 @@
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyAccountChooser)
 typedef struct {
-	EmpathyAccountManager          *manager;
+	TpAccountManager               *manager;
 	gboolean                        set_active_item;
 	gboolean			account_manually_set;
 	gboolean                        has_all_option;
 	EmpathyAccountChooserFilterFunc filter;
 	gpointer                        filter_data;
+	gboolean                        ready;
 } EmpathyAccountChooserPriv;
 
 typedef struct {
 	EmpathyAccountChooser *chooser;
-	EmpathyAccount        *account;
+	TpAccount             *account;
 	gboolean               set;
 } SetAccountData;
 
+/* Distinguishes between store entries which are actually accounts, and special
+ * items like the "All" entry and the separator below it, so they can be sorted
+ * correctly. Higher-numbered entries will sort earlier.
+ */
+typedef enum {
+	ROW_ACCOUNT = 0,
+	ROW_SEPARATOR,
+	ROW_ALL
+} RowType;
+
 enum {
 	COL_ACCOUNT_IMAGE,
 	COL_ACCOUNT_TEXT,
 	COL_ACCOUNT_ENABLED, /* Usually tied to connected state */
+	COL_ACCOUNT_ROW_TYPE,
 	COL_ACCOUNT_POINTER,
 	COL_ACCOUNT_COUNT
 };
@@ -86,24 +103,26 @@ static void     account_chooser_set_property           (GObject
 							const GValue             *value,
 							GParamSpec               *pspec);
 static void     account_chooser_setup                  (EmpathyAccountChooser    *chooser);
-static void     account_chooser_account_created_cb     (EmpathyAccountManager    *manager,
-							EmpathyAccount           *account,
+static void     account_chooser_account_validity_changed_cb (TpAccountManager    *manager,
+							TpAccount                *account,
+							gboolean                  valid,
 							EmpathyAccountChooser    *chooser);
-static void     account_chooser_account_add_foreach    (EmpathyAccount                *account,
+static void     account_chooser_account_add_foreach    (TpAccount                *account,
 							EmpathyAccountChooser    *chooser);
-static void     account_chooser_account_deleted_cb     (EmpathyAccountManager    *manager,
-							EmpathyAccount           *account,
+static void     account_chooser_account_removed_cb     (TpAccountManager         *manager,
+							TpAccount                *account,
 							EmpathyAccountChooser    *chooser);
-static void     account_chooser_account_remove_foreach (EmpathyAccount                *account,
+static void     account_chooser_account_remove_foreach (TpAccount                *account,
 							EmpathyAccountChooser    *chooser);
 static void     account_chooser_update_iter            (EmpathyAccountChooser    *chooser,
 							GtkTreeIter              *iter);
-static void     account_chooser_connection_changed_cb  (EmpathyAccountManager    *manager,
-							EmpathyAccount                *account,
-							TpConnectionStatusReason  reason,
-							TpConnectionStatus        new_status,
-							TpConnectionStatus        old_status,
-							EmpathyAccountChooser    *chooser);
+static void     account_chooser_status_changed_cb      (TpAccount  *account,
+							guint       old_status,
+							guint       new_status,
+							guint       reason,
+							gchar      *dbus_error_name,
+							GHashTable *details,
+							gpointer    user_data);
 static gboolean account_chooser_separator_func         (GtkTreeModel             *model,
 							GtkTreeIter              *iter,
 							EmpathyAccountChooser    *chooser);
@@ -117,6 +136,13 @@ enum {
 	PROP_HAS_ALL_OPTION,
 };
 
+enum {
+	READY,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 G_DEFINE_TYPE (EmpathyAccountChooser, empathy_account_chooser, GTK_TYPE_COMBO_BOX);
 
 static void
@@ -141,6 +167,16 @@ empathy_account_chooser_class_init (EmpathyAccountChooserClass *klass)
 							       FALSE,
 							       G_PARAM_READWRITE));
 
+	signals[READY] =
+		g_signal_new ("ready",
+			      G_OBJECT_CLASS_TYPE (object_class),
+			      G_SIGNAL_RUN_LAST,
+			      0,
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__VOID,
+			      G_TYPE_NONE,
+			      0);
+
 	g_type_class_add_private (object_class, sizeof (EmpathyAccountChooserPriv));
 }
 
@@ -156,16 +192,13 @@ empathy_account_chooser_init (EmpathyAccountChooser *chooser)
 	priv->filter = NULL;
 	priv->filter_data = NULL;
 
-	priv->manager = empathy_account_manager_dup_singleton ();
+	priv->manager = tp_account_manager_dup ();
 
-	g_signal_connect (priv->manager, "account-created",
-			  G_CALLBACK (account_chooser_account_created_cb),
+	g_signal_connect (priv->manager, "account-validity-changed",
+			  G_CALLBACK (account_chooser_account_validity_changed_cb),
 			  chooser);
-	g_signal_connect (priv->manager, "account-deleted",
-			  G_CALLBACK (account_chooser_account_deleted_cb),
-			  chooser);
-	g_signal_connect (priv->manager, "account-connection-changed",
-			  G_CALLBACK (account_chooser_connection_changed_cb),
+	g_signal_connect (priv->manager, "account-removed",
+			  G_CALLBACK (account_chooser_account_removed_cb),
 			  chooser);
 
 	account_chooser_setup (EMPATHY_ACCOUNT_CHOOSER (chooser));
@@ -177,13 +210,10 @@ account_chooser_finalize (GObject *object)
 	EmpathyAccountChooserPriv *priv = GET_PRIV (object);
 
 	g_signal_handlers_disconnect_by_func (priv->manager,
-					      account_chooser_connection_changed_cb,
+					      account_chooser_account_validity_changed_cb,
 					      object);
 	g_signal_handlers_disconnect_by_func (priv->manager,
-					      account_chooser_account_created_cb,
-					      object);
-	g_signal_handlers_disconnect_by_func (priv->manager,
-					      account_chooser_account_deleted_cb,
+					      account_chooser_account_removed_cb,
 					      object);
 	g_object_unref (priv->manager);
 
@@ -253,16 +283,16 @@ empathy_account_chooser_new (void)
  * @chooser: an #EmpathyAccountChooser
  *
  * Returns the account which is currently selected in the chooser or %NULL
- * if there is no account selected. The #EmpathyAccount returned should be
+ * if there is no account selected. The #TpAccount returned should be
  * unrefed with g_object_unref() when finished with.
  *
- * Return value: a new ref to the #EmpathyAccount currently selected, or %NULL.
+ * Return value: a new ref to the #TpAccount currently selected, or %NULL.
  */
-EmpathyAccount *
+TpAccount *
 empathy_account_chooser_dup_account (EmpathyAccountChooser *chooser)
 {
 	EmpathyAccountChooserPriv *priv;
-	EmpathyAccount           *account;
+	TpAccount                 *account;
 	GtkTreeModel             *model;
 	GtkTreeIter               iter;
 
@@ -295,7 +325,7 @@ TpConnection *
 empathy_account_chooser_get_connection (EmpathyAccountChooser *chooser)
 {
 	EmpathyAccountChooserPriv *priv;
-	EmpathyAccount            *account;
+	TpAccount                 *account;
 	TpConnection              *connection;
 
 	g_return_val_if_fail (EMPATHY_IS_ACCOUNT_CHOOSER (chooser), NULL);
@@ -303,7 +333,15 @@ empathy_account_chooser_get_connection (EmpathyAccountChooser *chooser)
 	priv = GET_PRIV (chooser);
 
 	account = empathy_account_chooser_dup_account (chooser);
-	connection = empathy_account_get_connection (account);
+
+	/* if the returned account is NULL, then the account manager probably
+	 * hasn't been prepared yet. It should be safe to return NULL here
+	 * though. */
+	if (account == NULL) {
+		return NULL;
+	}
+
+	connection = tp_account_get_connection (account);
 	g_object_unref (account);
 
 	return connection;
@@ -312,7 +350,7 @@ empathy_account_chooser_get_connection (EmpathyAccountChooser *chooser)
 /**
  * empathy_account_chooser_set_account:
  * @chooser: an #EmpathyAccountChooser
- * @account: an #EmpathyAccount
+ * @account: a #TpAccount
  *
  * Sets the currently selected account to @account, if it exists in the list.
  *
@@ -320,7 +358,7 @@ empathy_account_chooser_get_connection (EmpathyAccountChooser *chooser)
  */
 gboolean
 empathy_account_chooser_set_account (EmpathyAccountChooser *chooser,
-				     EmpathyAccount *account)
+				     TpAccount             *account)
 {
 	EmpathyAccountChooserPriv *priv;
 	GtkComboBox    *combobox;
@@ -417,6 +455,7 @@ empathy_account_chooser_set_has_all_option (EmpathyAccountChooser *chooser,
 				    COL_ACCOUNT_TEXT, NULL,
 				    COL_ACCOUNT_ENABLED, TRUE,
 				    COL_ACCOUNT_POINTER, NULL,
+				    COL_ACCOUNT_ROW_TYPE, ROW_SEPARATOR,
 				    -1);
 
 		gtk_list_store_prepend (store, &iter);
@@ -424,6 +463,7 @@ empathy_account_chooser_set_has_all_option (EmpathyAccountChooser *chooser,
 				    COL_ACCOUNT_TEXT, _("All"),
 				    COL_ACCOUNT_ENABLED, TRUE,
 				    COL_ACCOUNT_POINTER, NULL,
+				    COL_ACCOUNT_ROW_TYPE, ROW_ALL,
 				    -1);
 	} else {
 		if (gtk_tree_model_get_iter_first (model, &iter)) {
@@ -442,11 +482,93 @@ empathy_account_chooser_set_has_all_option (EmpathyAccountChooser *chooser,
 	g_object_notify (G_OBJECT (chooser), "has-all-option");
 }
 
+static void
+account_manager_prepared_cb (GObject *source_object,
+			     GAsyncResult *result,
+			     gpointer user_data)
+{
+	GList *accounts, *l;
+	TpAccountManager *manager = TP_ACCOUNT_MANAGER (source_object);
+	EmpathyAccountChooser *chooser = user_data;
+	EmpathyAccountChooserPriv *priv = GET_PRIV (chooser);
+	GError *error = NULL;
+
+	if (!tp_account_manager_prepare_finish (manager, result, &error)) {
+		DEBUG ("Failed to prepare account manager: %s", error->message);
+		g_error_free (error);
+		return;
+	}
+
+	accounts = tp_account_manager_get_valid_accounts (manager);
+
+	for (l = accounts; l != NULL; l = l->next) {
+		TpAccount *account = l->data;
+
+		account_chooser_account_add_foreach (account, chooser);
+
+		tp_g_signal_connect_object (account, "status-changed",
+					     G_CALLBACK (account_chooser_status_changed_cb),
+					     chooser, 0);
+	}
+
+	g_list_free (accounts);
+
+	priv->ready = TRUE;
+	g_signal_emit (chooser, signals[READY], 0);
+}
+
+static gint
+account_cmp (GtkTreeModel *model,
+	     GtkTreeIter *a,
+	     GtkTreeIter *b,
+	     gpointer user_data)
+{
+	RowType a_type, b_type;
+	gboolean a_enabled, b_enabled;
+	gchar *a_text, *b_text;
+	gint result;
+
+	gtk_tree_model_get (model, a,
+		COL_ACCOUNT_ENABLED, &a_enabled,
+		COL_ACCOUNT_ROW_TYPE, &a_type,
+		-1);
+	gtk_tree_model_get (model, b,
+		COL_ACCOUNT_ENABLED, &b_enabled,
+		COL_ACCOUNT_ROW_TYPE, &b_type,
+		-1);
+
+	/* This assumes that we have at most one of each special row type. */
+	if (a_type != b_type) {
+		/* Display higher-numbered special row types first. */
+		return (b_type - a_type);
+	}
+
+	/* Enabled accounts are displayed first */
+	if (a_enabled != b_enabled)
+		return a_enabled ? -1: 1;
+
+	gtk_tree_model_get (model, a, COL_ACCOUNT_TEXT, &a_text, -1);
+	gtk_tree_model_get (model, b, COL_ACCOUNT_TEXT, &b_text, -1);
+
+	if (a_text == b_text)
+		result = 0;
+	else if (a_text == NULL)
+		result = 1;
+	else if (b_text == NULL)
+		result = -1;
+	else
+		result = g_ascii_strcasecmp (a_text, b_text);
+
+	g_free (a_text);
+	g_free (b_text);
+
+	return result;
+}
+
 static void
 account_chooser_setup (EmpathyAccountChooser *chooser)
 {
 	EmpathyAccountChooserPriv *priv;
-	GList                    *accounts;
 	GtkListStore             *store;
 	GtkCellRenderer          *renderer;
 	GtkComboBox              *combobox;
@@ -462,7 +584,13 @@ account_chooser_setup (EmpathyAccountChooser *chooser)
 				    G_TYPE_STRING,    /* Image */
 				    G_TYPE_STRING,    /* Name */
 				    G_TYPE_BOOLEAN,   /* Enabled */
-				    EMPATHY_TYPE_ACCOUNT);
+				    G_TYPE_UINT,      /* Row type */
+				    TP_TYPE_ACCOUNT);
+
+	gtk_tree_sortable_set_default_sort_func (GTK_TREE_SORTABLE (store),
+		account_cmp, chooser, NULL);
+	gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
+		GTK_TREE_SORTABLE_DEFAULT_SORT_COLUMN_ID, GTK_SORT_ASCENDING);
 
 	gtk_combo_box_set_model (combobox, GTK_TREE_MODEL (store));
 
@@ -482,25 +610,27 @@ account_chooser_setup (EmpathyAccountChooser *chooser)
 					NULL);
 
 	/* Populate accounts */
-	accounts = empathy_account_manager_dup_accounts (priv->manager);
-	g_list_foreach (accounts,
-			(GFunc) account_chooser_account_add_foreach,
-			chooser);
+	tp_account_manager_prepare_async (priv->manager, NULL,
+					  account_manager_prepared_cb, chooser);
 
-	g_list_free (accounts);
 	g_object_unref (store);
 }
 
 static void
-account_chooser_account_created_cb (EmpathyAccountManager *manager,
-				    EmpathyAccount        *account,
-				    EmpathyAccountChooser *chooser)
+account_chooser_account_validity_changed_cb (TpAccountManager      *manager,
+					     TpAccount             *account,
+					     gboolean               valid,
+					     EmpathyAccountChooser *chooser)
 {
-	account_chooser_account_add_foreach (account, chooser);
+	if (valid) {
+		account_chooser_account_add_foreach (account, chooser);
+	} else {
+		account_chooser_account_remove_foreach (account, chooser);
+	}
 }
 
 static void
-account_chooser_account_add_foreach (EmpathyAccount        *account,
+account_chooser_account_add_foreach (TpAccount             *account,
 				     EmpathyAccountChooser *chooser)
 {
 	GtkListStore *store;
@@ -516,20 +646,18 @@ account_chooser_account_add_foreach (EmpathyAccount        *account,
 					   COL_ACCOUNT_POINTER, account,
 					   -1);
 	account_chooser_update_iter (chooser, &iter);
-	/* We got a reffed account and it was reffed by the liststore as well */
-	g_object_unref (account);
 }
 
 static void
-account_chooser_account_deleted_cb (EmpathyAccountManager *manager,
-				    EmpathyAccount        *account,
+account_chooser_account_removed_cb (TpAccountManager      *manager,
+				    TpAccount             *account,
 				    EmpathyAccountChooser *chooser)
 {
 	account_chooser_account_remove_foreach (account, chooser);
 }
 
 typedef struct {
-	EmpathyAccount   *account;
+	TpAccount   *account;
 	GtkTreeIter *iter;
 	gboolean     found;
 } FindAccountData;
@@ -541,7 +669,7 @@ account_chooser_find_account_foreach (GtkTreeModel *model,
 				      gpointer      user_data)
 {
 	FindAccountData *data = user_data;
-	EmpathyAccount  *account;
+	TpAccount  *account;
 
 	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
 
@@ -560,7 +688,7 @@ account_chooser_find_account_foreach (GtkTreeModel *model,
 
 static gboolean
 account_chooser_find_account (EmpathyAccountChooser *chooser,
-			      EmpathyAccount        *account,
+			      TpAccount             *account,
 			      GtkTreeIter           *iter)
 {
 	GtkListStore    *store;
@@ -580,7 +708,7 @@ account_chooser_find_account (EmpathyAccountChooser *chooser,
 }
 
 static void
-account_chooser_account_remove_foreach (EmpathyAccount        *account,
+account_chooser_account_remove_foreach (TpAccount             *account,
 					EmpathyAccountChooser *chooser)
 {
 	GtkListStore *store;
@@ -602,7 +730,7 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 	EmpathyAccountChooserPriv *priv;
 	GtkListStore              *store;
 	GtkComboBox               *combobox;
-	EmpathyAccount            *account;
+	TpAccount                 *account;
 	const gchar               *icon_name;
 	gboolean                   is_enabled = TRUE;
 
@@ -615,14 +743,18 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 			    COL_ACCOUNT_POINTER, &account,
 			    -1);
 
-	icon_name = empathy_account_get_icon_name (account);
+	/* Skip rows without account associated */
+	if (account == NULL)
+		return;
+
+	icon_name = tp_account_get_icon_name (account);
 	if (priv->filter) {
 		is_enabled = priv->filter (account, priv->filter_data);
 	}
 
 	gtk_list_store_set (store, iter,
 			    COL_ACCOUNT_IMAGE, icon_name,
-			    COL_ACCOUNT_TEXT, empathy_account_get_display_name (account),
+			    COL_ACCOUNT_TEXT, tp_account_get_display_name (account),
 			    COL_ACCOUNT_ENABLED, is_enabled,
 			    -1);
 
@@ -637,13 +769,15 @@ account_chooser_update_iter (EmpathyAccountChooser *chooser,
 }
 
 static void
-account_chooser_connection_changed_cb (EmpathyAccountManager   *manager,
-				       EmpathyAccount          *account,
-				       TpConnectionStatusReason reason,
-				       TpConnectionStatus       new_status,
-				       TpConnectionStatus       old_status,
-				       EmpathyAccountChooser   *chooser)
+account_chooser_status_changed_cb (TpAccount  *account,
+				   guint       old_status,
+				   guint       new_status,
+				   guint       reason,
+				   gchar      *dbus_error_name,
+				   GHashTable *details,
+				   gpointer    user_data)
 {
+	EmpathyAccountChooser *chooser = user_data;
 	GtkTreeIter iter;
 
 	if (account_chooser_find_account (chooser, account, &iter)) {
@@ -656,21 +790,10 @@ account_chooser_separator_func (GtkTreeModel         *model,
 				GtkTreeIter          *iter,
 				EmpathyAccountChooser *chooser)
 {
-	EmpathyAccountChooserPriv *priv;
-	gchar                    *text;
-	gboolean                  is_separator;
+	RowType row_type;
 
-	priv = GET_PRIV (chooser);
-
-	if (!priv->has_all_option) {
-		return FALSE;
-	}
-
-	gtk_tree_model_get (model, iter, COL_ACCOUNT_TEXT, &text, -1);
-	is_separator = text == NULL;
-	g_free (text);
-
-	return is_separator;
+	gtk_tree_model_get (model, iter, COL_ACCOUNT_ROW_TYPE, &row_type, -1);
+	return (row_type == ROW_SEPARATOR);
 }
 
 static gboolean
@@ -679,18 +802,12 @@ account_chooser_set_account_foreach (GtkTreeModel   *model,
 				     GtkTreeIter    *iter,
 				     SetAccountData *data)
 {
-	EmpathyAccount *account;
+	TpAccount *account;
 	gboolean   equal;
 
 	gtk_tree_model_get (model, iter, COL_ACCOUNT_POINTER, &account, -1);
 
-	/* Special case so we can make it possible to select the All option */
-	if ((data->account == NULL) != (account == NULL)) {
-		equal = FALSE;
-	}
-	else {
-		equal = (data->account == account);
-	}
+	equal = (data->account == account);
 
 	if (account) {
 		g_object_unref (account);
@@ -750,7 +867,7 @@ empathy_account_chooser_set_filter (EmpathyAccountChooser           *chooser,
 
 /**
  * EmpathyAccountChooserFilterFunc:
- * @account: an #EmpathyAccount
+ * @account: a #TpAccount
  * @user_data: user data, or %NULL
  *
  * A function which decides whether the account indicated by @account
@@ -761,7 +878,7 @@ empathy_account_chooser_set_filter (EmpathyAccountChooser           *chooser,
 
 /**
  * empathy_account_chooser_filter_is_connected:
- * @account: an #EmpathyAccount
+ * @account: a #TpAccount
  * @user_data: user data or %NULL
  *
  * A useful #EmpathyAccountChooserFilterFunc that one could pass into
@@ -770,13 +887,31 @@ empathy_account_chooser_set_filter (EmpathyAccountChooser           *chooser,
  * Return value: Whether @account is connected
  */
 gboolean
-empathy_account_chooser_filter_is_connected (EmpathyAccount *account,
+empathy_account_chooser_filter_is_connected (TpAccount *account,
 					     gpointer   user_data)
 {
-	TpConnectionStatus  status;
-
-	g_object_get (account, "connection-status", &status, NULL);
-
-	return status == TP_CONNECTION_STATUS_CONNECTED;
+	return (tp_account_get_connection_status (account, NULL)
+	    == TP_CONNECTION_STATUS_CONNECTED);
 }
 
+gboolean
+empathy_account_chooser_is_ready (EmpathyAccountChooser *self)
+{
+	EmpathyAccountChooserPriv *priv = GET_PRIV (self);
+
+	return priv->ready;
+}
+
+TpAccount *
+empathy_account_chooser_get_account (EmpathyAccountChooser *chooser)
+{
+	TpAccount *account;
+
+	account = empathy_account_chooser_dup_account (chooser);
+	if (account == NULL)
+		return NULL;
+
+	g_object_unref (account);
+
+	return account;
+}
