@@ -36,18 +36,13 @@
 
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/util.h>
-#ifdef ENABLE_TPL
 #include <telepathy-logger/log-manager.h>
-#else
-
-#include <libempathy/empathy-log-manager.h>
-#endif /* ENABLE_TPL */
 #include <libempathy/empathy-contact-list.h>
+#include <libempathy/empathy-gsettings.h>
 #include <libempathy/empathy-utils.h>
 #include <libempathy/empathy-dispatcher.h>
 
 #include "empathy-chat.h"
-#include "empathy-conf.h"
 #include "empathy-spell.h"
 #include "empathy-contact-list-store.h"
 #include "empathy-contact-list-view.h"
@@ -68,7 +63,7 @@
 #define COMPOSING_STOP_TIMEOUT 5
 
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyChat)
-typedef struct {
+struct _EmpathyChatPriv {
 	EmpathyTpChat     *tp_chat;
 	TpAccount         *account;
 	gchar             *id;
@@ -77,11 +72,10 @@ typedef struct {
 	EmpathyContact    *remote_contact;
 	gboolean           show_contacts;
 
-#ifdef ENABLE_TPL
+	GSettings         *gsettings_chat;
+	GSettings         *gsettings_ui;
+
 	TplLogManager     *log_manager;
-#else
-	EmpathyLogManager *log_manager;
-#endif /* ENABLE_TPL */
 	TpAccountManager  *account_manager;
 	GList             *input_history;
 	GList             *input_history_current;
@@ -104,10 +98,6 @@ typedef struct {
 	gulong		   delete_range_id;
 	gulong		   notify_cursor_position_id;
 
-	/* This stores the id for the spell checking configuration setting
-	 * notification signal handler. */
-	guint		   conf_notify_id;
-
 	GtkWidget         *widget;
 	GtkWidget         *hpaned;
 	GtkWidget         *vbox_left;
@@ -148,7 +138,7 @@ typedef struct {
 	 * notified again about the already notified pending messages when the
 	 * messages in tab will be properly shown */
 	gboolean           retrieving_backlogs;
-} EmpathyChatPriv;
+};
 
 typedef struct {
 	gchar *text; /* Original message that was specified
@@ -1559,8 +1549,8 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 		 */
 
 		view = GTK_TEXT_VIEW (chat->input_text_view);
-		if (gtk_im_context_filter_keypress (view->im_context, event)) {
-			GTK_TEXT_VIEW (chat->input_text_view)->need_im_reset = TRUE;
+		if (gtk_text_view_im_context_filter_keypress (view, event)) {
+			gtk_text_view_reset_im_context (view);
 			return TRUE;
 		}
 
@@ -1623,7 +1613,6 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 		if (completed) {
 			guint        len;
 			const gchar *text;
-			gchar       *complete_char = NULL;
 			GString     *message = NULL;
 			GList       *l;
 
@@ -1657,16 +1646,20 @@ chat_input_key_press_event_cb (GtkWidget   *widget,
 
 			gtk_text_buffer_insert_at_cursor (buffer, text, strlen (text));
 
-			if (len == 1 && is_start_of_buffer &&
-			    empathy_conf_get_string (empathy_conf_get (),
-						     EMPATHY_PREFS_CHAT_NICK_COMPLETION_CHAR,
-						     &complete_char) &&
-			    complete_char != NULL) {
+			if (len == 1 && is_start_of_buffer) {
+			    gchar *complete_char;
+
+			    complete_char = g_settings_get_string (
+				    priv->gsettings_chat,
+				    EMPATHY_PREFS_CHAT_NICK_COMPLETION_CHAR);
+
+			    if (complete_char != NULL) {
 				gtk_text_buffer_insert_at_cursor (buffer,
 								  complete_char,
 								  strlen (complete_char));
 				gtk_text_buffer_insert_at_cursor (buffer, " ", 1);
 				g_free (complete_char);
+			    }
 			}
 
 			g_free (completed);
@@ -1928,43 +1921,28 @@ chat_input_populate_popup_cb (GtkTextView *view,
 }
 
 
-#ifdef ENABLE_TPL
 static gboolean
-chat_log_filter (TplLogEntry *log,
+chat_log_filter (TplEntry *log,
 		 gpointer user_data)
-#else
-static gboolean
-chat_log_filter (EmpathyMessage *message,
-		 gpointer user_data)
-#endif /* ENABLE_TPL */
 {
 	EmpathyChat *chat = user_data;
-#ifdef ENABLE_TPL
 	EmpathyMessage *message;
-#endif /* ENABLE_TPL */
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	const GList *pending;
 
-#ifdef ENABLE_TPL
-	g_return_val_if_fail (TPL_IS_LOG_ENTRY (log), FALSE);
-#else
-	g_return_val_if_fail (EMPATHY_IS_MESSAGE (message), FALSE);
-#endif /* ENABLE_TPL */
+	g_return_val_if_fail (TPL_IS_ENTRY (log), FALSE);
 	g_return_val_if_fail (EMPATHY_IS_CHAT (chat), FALSE);
 
 	pending = empathy_tp_chat_get_pending_messages (priv->tp_chat);
-#ifdef ENABLE_TPL
 	message = empathy_message_from_tpl_log_entry (log);
-#endif /* ENABLE_TPL */
 
 	for (; pending; pending = g_list_next (pending)) {
 		if (empathy_message_equal (message, pending->data)) {
 			return FALSE;
 		}
 	}
-#ifdef ENABLE_TPL
+
 	g_object_unref (message);
-#endif
 	return TRUE;
 }
 
@@ -1991,7 +1969,6 @@ show_pending_messages (EmpathyChat *chat) {
 }
 
 
-#ifdef ENABLE_TPL
 static void
 got_filtered_messages_cb (GObject *manager,
 		GAsyncResult *result,
@@ -2003,9 +1980,8 @@ got_filtered_messages_cb (GObject *manager,
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	GError *error = NULL;
 
-	messages = tpl_log_manager_get_filtered_messages_async_finish (result, &error);
-
-	if (error != NULL) {
+	if (!tpl_log_manager_get_filtered_messages_finish (TPL_LOG_MANAGER (manager),
+		result, &messages, &error)) {
 		DEBUG ("%s. Aborting.", error->message);
 		empathy_chat_view_append_event (chat->view,
 			_("Failed to retrieve recent logs"));
@@ -2015,7 +1991,7 @@ got_filtered_messages_cb (GObject *manager,
 
 	for (l = messages; l; l = g_list_next (l)) {
 		EmpathyMessage *message;
-		g_assert (TPL_IS_LOG_ENTRY (l->data));
+		g_assert (TPL_IS_ENTRY (l->data));
 
 		message = empathy_message_from_tpl_log_entry (l->data);
 		g_object_unref (l->data);
@@ -2038,17 +2014,12 @@ out:
 	/* Turn back on scrolling */
 	empathy_chat_view_scroll (chat->view, TRUE);
 }
-#endif /* ENABLE_TPL */
-
 
 static void
 chat_add_logs (EmpathyChat *chat)
 {
 	EmpathyChatPriv *priv = GET_PRIV (chat);
 	gboolean         is_chatroom;
-#ifndef ENABLE_TPL
-	GList           *messages, *l;
-#endif /* ENABLE_TPL */
 
 	if (!priv->id) {
 		return;
@@ -2060,25 +2031,6 @@ chat_add_logs (EmpathyChat *chat)
 	/* Add messages from last conversation */
 	is_chatroom = priv->handle_type == TP_HANDLE_TYPE_ROOM;
 
-#ifndef ENABLE_TPL
-	messages = empathy_log_manager_get_filtered_messages (priv->log_manager,
-							      priv->account,
-							      priv->id,
-							      is_chatroom,
-							      5,
-							      chat_log_filter,
-							      chat);
-
-	for (l = messages; l; l = g_list_next (l)) {
-		empathy_chat_view_append_message (chat->view, l->data);
-		g_object_unref (l->data);
-	}
-
-	g_list_free (messages);
-
-	/* Turn back on scrolling */
-	empathy_chat_view_scroll (chat->view, TRUE);
-#else
 	priv->retrieving_backlogs = TRUE;
 	tpl_log_manager_get_filtered_messages_async (priv->log_manager,
 							      priv->account,
@@ -2089,7 +2041,6 @@ chat_add_logs (EmpathyChat *chat)
 							      chat,
 							      got_filtered_messages_cb,
 							      (gpointer) chat);
-#endif /* ENABLE_TPL */
 }
 
 static gint
@@ -2386,7 +2337,7 @@ update_misspelled_words (gpointer data)
 }
 
 static void
-conf_spell_checking_cb (EmpathyConf *conf,
+conf_spell_checking_cb (GSettings *gsettings_chat,
 			const gchar *key,
 			gpointer user_data)
 {
@@ -2398,8 +2349,8 @@ conf_spell_checking_cb (EmpathyConf *conf,
 	if (strcmp (key, EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED) != 0)
 		return;
 
-	empathy_conf_get_bool (conf, EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
-                               &spell_checker);
+	spell_checker = g_settings_get_boolean (gsettings_chat,
+			EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED);
 
 	if (!empathy_spell_supported ()) {
 		spell_checker = FALSE;
@@ -2469,13 +2420,18 @@ conf_spell_checking_cb (EmpathyConf *conf,
 }
 
 static gboolean
-chat_hpaned_pos_changed_cb (GtkWidget* hpaned, gpointer user_data)
+chat_hpaned_pos_changed_cb (GtkWidget* hpaned,
+		GParamSpec *spec,
+		gpointer user_data)
 {
+	EmpathyChat *chat = EMPATHY_CHAT (user_data);
 	gint hpaned_pos;
+
 	hpaned_pos = gtk_paned_get_position (GTK_PANED(hpaned));
-	empathy_conf_set_int (empathy_conf_get (),
-			      EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
-			      hpaned_pos);
+	g_settings_set_int (chat->priv->gsettings_ui,
+			    EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
+			    hpaned_pos);
+
 	return TRUE;
 }
 
@@ -2552,11 +2508,10 @@ chat_create_ui (EmpathyChat *chat)
 	tp_g_signal_connect_object  (buffer, "changed",
 			  G_CALLBACK (chat_input_text_buffer_changed_cb),
 			  chat, 0);
-	priv->conf_notify_id =
-			empathy_conf_notify_add (empathy_conf_get (),
-						 EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
-						 conf_spell_checking_cb, chat);
-	conf_spell_checking_cb (empathy_conf_get (),
+	tp_g_signal_connect_object (priv->gsettings_chat,
+			"changed::" EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED,
+			G_CALLBACK (conf_spell_checking_cb), chat, 0);
+	conf_spell_checking_cb (priv->gsettings_chat,
 				EMPATHY_PREFS_CHAT_SPELL_CHECKER_ENABLED, chat);
 	gtk_container_add (GTK_CONTAINER (priv->scrolled_window_input),
 			   chat->input_text_view);
@@ -2574,13 +2529,12 @@ chat_create_ui (EmpathyChat *chat)
 
 	g_signal_connect (priv->hpaned, "notify::position",
 			  G_CALLBACK (chat_hpaned_pos_changed_cb),
-			  NULL);
+			  chat);
 
         /* Load the paned position */
-	if (empathy_conf_get_int (empathy_conf_get (),
-				 EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS,
-				 &paned_pos)
-		&& paned_pos)
+	paned_pos = g_settings_get_int (priv->gsettings_ui,
+			EMPATHY_PREFS_UI_CHAT_WINDOW_PANED_POS);
+	if (paned_pos != 0)
 		gtk_paned_set_position (GTK_PANED(priv->hpaned), paned_pos);
 
 	/* Set widget focus order */
@@ -2661,11 +2615,8 @@ chat_finalize (GObject *object)
 
 	DEBUG ("Finalized: %p", object);
 
-	if (priv->conf_notify_id != 0) {
-		empathy_conf_notify_remove (empathy_conf_get (),
-					    priv->conf_notify_id);
-		priv->conf_notify_id = 0;
-	}
+	g_object_unref (priv->gsettings_chat);
+	g_object_unref (priv->gsettings_ui);
 
 	g_list_foreach (priv->input_history, (GFunc) chat_input_history_entry_free, NULL);
 	g_list_free (priv->input_history);
@@ -2723,12 +2674,6 @@ chat_constructed (GObject *object)
 
 	if (priv->handle_type != TP_HANDLE_TYPE_ROOM)
 		chat_add_logs (chat);
-#ifndef ENABLE_TPL
-	/* When async API are involved, pending message are shown at the end of the
-	 * callbacks' chain fired by chat_add_logs */
-	priv->can_show_pending = TRUE;
-	show_pending_messages (chat);
-#endif /* ENABLE_TPL */
 }
 
 static void
@@ -2871,11 +2816,10 @@ empathy_chat_init (EmpathyChat *chat)
 		EMPATHY_TYPE_CHAT, EmpathyChatPriv);
 
 	chat->priv = priv;
-#ifndef ENABLE_TPL
-	priv->log_manager = empathy_log_manager_dup_singleton ();
-#else
 	priv->log_manager = tpl_log_manager_dup_singleton ();
-#endif /* ENABLE_TPL */
+	priv->gsettings_chat = g_settings_new (EMPATHY_PREFS_CHAT_SCHEMA);
+	priv->gsettings_ui = g_settings_new (EMPATHY_PREFS_UI_SCHEMA);
+
 	priv->contacts_width = -1;
 	priv->input_history = NULL;
 	priv->input_history_current = NULL;
@@ -2884,9 +2828,8 @@ empathy_chat_init (EmpathyChat *chat)
 	tp_account_manager_prepare_async (priv->account_manager, NULL,
 					  account_manager_prepared_cb, chat);
 
-	empathy_conf_get_bool (empathy_conf_get (),
-			       EMPATHY_PREFS_CHAT_SHOW_CONTACTS_IN_ROOMS,
-			       &priv->show_contacts);
+	priv->show_contacts = g_settings_get_boolean (priv->gsettings_chat,
+			EMPATHY_PREFS_CHAT_SHOW_CONTACTS_IN_ROOMS);
 
 	/* Block events for some time to avoid having "has come online" or
 	 * "joined" messages. */
