@@ -790,48 +790,6 @@ empathy_contact_list_store_get_parent_group (GtkTreeModel *model,
 	return name;
 }
 
-gboolean
-empathy_contact_list_store_search_equal_func (GtkTreeModel *model,
-					      gint          column,
-					      const gchar  *key,
-					      GtkTreeIter  *iter,
-					      gpointer      search_data)
-{
-	gchar    *name, *name_folded;
-	gchar    *key_folded;
-	gboolean  ret;
-
-	g_return_val_if_fail (GTK_IS_TREE_MODEL (model), FALSE);
-
-	if (!key) {
-		return TRUE;
-	}
-
-	gtk_tree_model_get (model, iter,
-			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, &name,
-			    -1);
-
-	if (!name) {
-		return TRUE;
-	}
-
-	name_folded = g_utf8_casefold (name, -1);
-	key_folded = g_utf8_casefold (key, -1);
-
-	if (name_folded && key_folded &&
-	    strstr (name_folded, key_folded)) {
-		ret = FALSE;
-	} else {
-		ret = TRUE;
-	}
-
-	g_free (name);
-	g_free (name_folded);
-	g_free (key_folded);
-
-	return ret;
-}
-
 static void
 contact_list_store_setup (EmpathyContactListStore *store)
 {
@@ -1019,10 +977,11 @@ contact_list_store_groups_changed_cb (EmpathyContactList      *list_iface,
 static void
 add_contact_to_store (GtkTreeStore *store,
 		      GtkTreeIter *iter,
+		      GtkTreeIter *parent,
 		      EmpathyContact *contact,
 		      EmpathyContactListFlags flags)
 {
-	gtk_tree_store_set (store, iter,
+	gtk_tree_store_insert_with_values (store, iter, parent, 0,
 			    EMPATHY_CONTACT_LIST_STORE_COL_NAME, empathy_contact_get_name (contact),
 			    EMPATHY_CONTACT_LIST_STORE_COL_CONTACT, contact,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, FALSE,
@@ -1085,10 +1044,7 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 				&iter_group, NULL, NULL, TRUE);
 		}
 
-		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
-					     parent, NULL);
-
-		add_contact_to_store (GTK_TREE_STORE (store), &iter,
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, parent,
 				      contact, flags);
 	}
 
@@ -1100,15 +1056,11 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 
 		contact_list_store_get_group (store, l->data, &iter_group, NULL, NULL, FALSE);
 
-		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
-					     &iter_group, NULL);
-
-		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, &iter_group, contact, flags);
 		g_free (l->data);
 	}
 	g_list_free (groups);
 
-#ifdef HAVE_FAVOURITE_CONTACTS
 	if (priv->show_groups &&
 	    empathy_contact_list_is_favourite (priv->list, contact)) {
 	/* Add contact to the fake 'Favorites' group */
@@ -1117,12 +1069,8 @@ contact_list_store_add_contact (EmpathyContactListStore *store,
 		contact_list_store_get_group (store, EMPATHY_CONTACT_LIST_STORE_FAVORITE,
 			&iter_group, NULL, NULL, TRUE);
 
-		gtk_tree_store_insert_after (GTK_TREE_STORE (store), &iter,
-					     &iter_group, NULL);
-
-		add_contact_to_store (GTK_TREE_STORE (store), &iter, contact, flags);
+		add_contact_to_store (GTK_TREE_STORE (store), &iter, &iter_group, contact, flags);
 	}
-#endif
 
 	contact_list_store_contact_update (store, contact);
 }
@@ -1486,8 +1434,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 			*created = TRUE;
 		}
 
-		gtk_tree_store_append (GTK_TREE_STORE (store), &iter_group, NULL);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &iter_group,
+		gtk_tree_store_insert_with_values (GTK_TREE_STORE (store), &iter_group, NULL, 0,
 				    EMPATHY_CONTACT_LIST_STORE_COL_ICON_STATUS, NULL,
 				    EMPATHY_CONTACT_LIST_STORE_COL_NAME, name,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_GROUP, TRUE,
@@ -1500,10 +1447,7 @@ contact_list_store_get_group (EmpathyContactListStore *store,
 			*iter_group_to_set = iter_group;
 		}
 
-		gtk_tree_store_append (GTK_TREE_STORE (store),
-				       &iter_separator,
-				       &iter_group);
-		gtk_tree_store_set (GTK_TREE_STORE (store), &iter_separator,
+		gtk_tree_store_insert_with_values (GTK_TREE_STORE (store), &iter_separator, &iter_group, 0,
 				    EMPATHY_CONTACT_LIST_STORE_COL_IS_SEPARATOR, TRUE,
 				    -1);
 
@@ -1627,6 +1571,8 @@ contact_list_store_contact_sort (EmpathyContact *contact_a,
 	TpAccount *account_a, *account_b;
 	gint ret_val;
 
+	g_return_val_if_fail (contact_a != NULL || contact_b != NULL, 0);
+
 	/* alias */
 	ret_val = g_utf8_collate (empathy_contact_get_name (contact_a),
 				  empathy_contact_get_name (contact_b));
@@ -1684,10 +1630,9 @@ contact_list_store_state_sort_func (GtkTreeModel *model,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_b,
 			    -1);
 
-	ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
-		name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
-
-	if (ret_val != 0) {
+	if (contact_a == NULL || contact_b == NULL) {
+		ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
+			name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
 		goto free_and_out;
 	}
 
@@ -1743,10 +1688,10 @@ contact_list_store_name_sort_func (GtkTreeModel *model,
 			    EMPATHY_CONTACT_LIST_STORE_COL_IS_FAKE_GROUP, &fake_group_b,
 			    -1);
 
-	ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
-		name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
-
-	if (ret_val == 0)
+	if (contact_a == NULL || contact_b == NULL)
+		ret_val = compare_separator_and_groups (is_separator_a, is_separator_b,
+			name_a, name_b, contact_a, contact_b, fake_group_a, fake_group_b);
+	else
 		ret_val = contact_list_store_contact_sort (contact_a, contact_b);
 
 	if (contact_a) {
