@@ -25,12 +25,8 @@
 
 #include <glib/gi18n-lib.h>
 #include <gtk/gtk.h>
-#ifdef ENABLE_TPL
 #include <telepathy-logger/log-manager.h>
-#else
 
-#include <libempathy/empathy-log-manager.h>
-#endif /* ENABLE_TPL */
 #include <libempathy/empathy-call-factory.h>
 #include <libempathy/empathy-contact-manager.h>
 #include <libempathy/empathy-dispatcher.h>
@@ -135,14 +131,12 @@ empathy_contact_menu_new (EmpathyContact             *contact,
 		gtk_widget_show (item);
 	}
 
-#if HAVE_FAVOURITE_CONTACTS
 	/* Favorite checkbox */
 	if (features & EMPATHY_CONTACT_FEATURE_FAVOURITE) {
 		item = empathy_contact_favourite_menu_item_new (contact);
 		gtk_menu_shell_append (shell, item);
 		gtk_widget_show (item);
 	}
-#endif
 
 	return menu;
 }
@@ -254,7 +248,8 @@ empathy_contact_audio_call_menu_item_activated (GtkMenuItem *item,
 	EmpathyCallFactory *factory;
 
 	factory = empathy_call_factory_get ();
-	empathy_call_factory_new_call_with_streams (factory, contact, TRUE, FALSE);
+	empathy_call_factory_new_call_with_streams (factory, contact, TRUE, FALSE,
+		gtk_get_current_event_time (), NULL, NULL);
 }
 
 GtkWidget *
@@ -286,7 +281,8 @@ empathy_contact_video_call_menu_item_activated (GtkMenuItem *item,
 	EmpathyCallFactory *factory;
 
 	factory = empathy_call_factory_get ();
-	empathy_call_factory_new_call_with_streams (factory, contact, TRUE, TRUE);
+	empathy_call_factory_new_call_with_streams (factory, contact, TRUE, TRUE,
+		gtk_get_current_event_time (), NULL, NULL);
 }
 
 GtkWidget *
@@ -322,30 +318,19 @@ contact_log_menu_item_activate_cb (EmpathyContact *contact)
 GtkWidget *
 empathy_contact_log_menu_item_new (EmpathyContact *contact)
 {
-#ifndef ENABLE_TPL
-	EmpathyLogManager *manager;
-#else
 	TplLogManager     *manager;
-#endif /* ENABLE_TPL */
 	gboolean           have_log;
 	GtkWidget         *item;
 	GtkWidget         *image;
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
-#ifndef ENABLE_TPL
-	manager = empathy_log_manager_dup_singleton ();
-	have_log = empathy_log_manager_exists (manager,
-					       empathy_contact_get_account (contact),
-					       empathy_contact_get_id (contact),
-					       FALSE);
-#else
 	manager = tpl_log_manager_dup_singleton ();
 	have_log = tpl_log_manager_exists (manager,
 					       empathy_contact_get_account (contact),
 					       empathy_contact_get_id (contact),
 					       FALSE);
-#endif /* ENABLE_TPL */
+
 	g_object_unref (manager);
 
 	item = gtk_image_menu_item_new_with_mnemonic (_("_Previous Conversations"));
@@ -370,7 +355,7 @@ empathy_contact_file_transfer_menu_item_new (EmpathyContact *contact)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("Send file"));
+	item = gtk_image_menu_item_new_with_mnemonic (_("Send File"));
 	image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_DOCUMENT_SEND,
 					      GTK_ICON_SIZE_MENU);
 	gtk_widget_set_sensitive (item, empathy_contact_can_send_files (contact));
@@ -393,10 +378,11 @@ empathy_contact_share_my_desktop_menu_item_new (EmpathyContact *contact)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("Share my desktop"));
+	item = gtk_image_menu_item_new_with_mnemonic (_("Share My Desktop"));
 	image = gtk_image_new_from_icon_name (GTK_STOCK_NETWORK,
 					      GTK_ICON_SIZE_MENU);
-	gtk_widget_set_sensitive (item, empathy_contact_can_use_stream_tube (contact));
+	gtk_widget_set_sensitive (item, empathy_contact_can_use_rfb_stream_tube (
+		contact));
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
 	gtk_widget_show (image);
 
@@ -407,7 +393,6 @@ empathy_contact_share_my_desktop_menu_item_new (EmpathyContact *contact)
 	return item;
 }
 
-#if HAVE_FAVOURITE_CONTACTS
 static void
 favourite_menu_item_toggled_cb (GtkCheckMenuItem *item,
 	EmpathyContact *contact)
@@ -447,7 +432,6 @@ empathy_contact_favourite_menu_item_new (EmpathyContact *contact)
 	g_object_unref (manager);
 	return item;
 }
-#endif
 
 static void
 contact_info_menu_item_activate_cb (EmpathyContact *contact)
@@ -594,7 +578,7 @@ empathy_contact_invite_menu_item_new (EmpathyContact *contact)
 
 	g_return_val_if_fail (EMPATHY_IS_CONTACT (contact), NULL);
 
-	item = gtk_image_menu_item_new_with_mnemonic (_("_Invite to chat room"));
+	item = gtk_image_menu_item_new_with_mnemonic (_("_Invite to Chat Room"));
 	image = gtk_image_new_from_icon_name (EMPATHY_IMAGE_GROUP_MESSAGE,
 					      GTK_ICON_SIZE_MENU);
 	gtk_image_menu_item_set_image (GTK_IMAGE_MENU_ITEM (item), image);
