@@ -50,10 +50,12 @@ typedef struct _EmpathyContactSelectorDialogPriv \
           EmpathyContactSelectorDialogPriv;
 
 struct _EmpathyContactSelectorDialogPriv {
+  GtkListStore *store;
   GtkWidget *account_chooser_label;
   GtkWidget *account_chooser;
   GtkWidget *entry_id;
   EmpathyContactManager *contact_manager;
+  TpAccount *filter_account;
 
   gboolean show_account_chooser;
 };
@@ -64,7 +66,8 @@ struct _EmpathyContactSelectorDialogPriv {
 
 enum {
   PROP_0,
-  PROP_SHOW_ACCOUNT_CHOOSER
+  PROP_SHOW_ACCOUNT_CHOOSER,
+  PROP_FILTER_ACCOUNT
 };
 
 enum {
@@ -81,13 +84,9 @@ contact_selector_dialog_account_changed_cb (GtkWidget *widget,
   EmpathyAccountChooser *chooser;
   TpConnection *connection;
   GList *members;
-  GtkListStore *store;
-  GtkEntryCompletion *completion;
 
   /* Remove completions */
-  completion = gtk_entry_get_completion (GTK_ENTRY (priv->entry_id));
-  store = GTK_LIST_STORE (gtk_entry_completion_get_model (completion));
-  gtk_list_store_clear (store);
+  gtk_list_store_clear (priv->store);
 
   /* Get members of the new account */
   chooser = EMPATHY_ACCOUNT_CHOOSER (priv->account_chooser);
@@ -106,8 +105,25 @@ contact_selector_dialog_account_changed_cb (GtkWidget *widget,
     }
   else
     {
-      members = empathy_contact_list_get_members (
-          EMPATHY_CONTACT_LIST (priv->contact_manager));
+      if (priv->filter_account != NULL)
+        {
+          EmpathyTpContactList *contact_list;
+
+          connection = tp_account_get_connection (priv->filter_account);
+          if (connection == NULL)
+            return;
+
+          contact_list = empathy_contact_manager_get_list (
+              priv->contact_manager, connection);
+
+          members = empathy_contact_list_get_members (
+              EMPATHY_CONTACT_LIST (contact_list));
+        }
+      else
+        {
+          members = empathy_contact_list_get_members (
+              EMPATHY_CONTACT_LIST (priv->contact_manager));
+        }
     }
 
   /* Add members to the completion */
@@ -125,7 +141,7 @@ contact_selector_dialog_account_changed_cb (GtkWidget *widget,
         empathy_contact_get_name (contact),
         empathy_contact_get_id (contact));
 
-      gtk_list_store_insert_with_values (store, &iter, -1,
+      gtk_list_store_insert_with_values (priv->store, &iter, -1,
         COMPLETION_COL_TEXT, tmpstr,
         COMPLETION_COL_ID, empathy_contact_get_id (contact),
         COMPLETION_COL_NAME, empathy_contact_get_name (contact),
@@ -241,6 +257,28 @@ account_chooser_filter (TpAccount *account,
   return class->account_filter (self, account);
 }
 
+static gboolean
+contact_selector_dialog_filter_visible (GtkTreeModel *model,
+                                        GtkTreeIter  *iter,
+                                        gpointer      data)
+{
+  EmpathyContactSelectorDialog *self = EMPATHY_CONTACT_SELECTOR_DIALOG (data);
+  gboolean r;
+  char *id;
+
+  gtk_tree_model_get (model, iter,
+      COMPLETION_COL_ID, &id,
+      -1);
+
+  /* this must be non-NULL for this function to get called */
+  r = EMPATHY_CONTACT_SELECTOR_DIALOG_GET_CLASS (self)->contact_filter (
+      self, id);
+
+  g_free (id);
+
+  return r;
+}
+
 static void
 empathy_contact_selector_dialog_init (EmpathyContactSelectorDialog *dialog)
 {
@@ -248,7 +286,6 @@ empathy_contact_selector_dialog_init (EmpathyContactSelectorDialog *dialog)
   GtkBuilder *gui;
   gchar *filename;
   GtkEntryCompletion *completion;
-  GtkListStore *model;
   GtkWidget *content_area;
   GtkWidget *table_contact;
 
@@ -289,19 +326,20 @@ empathy_contact_selector_dialog_init (EmpathyContactSelectorDialog *dialog)
   gtk_container_set_border_width (GTK_CONTAINER (dialog), 6);
 
   /* text completion */
+  priv->store = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
+
   completion = gtk_entry_completion_new ();
-  model = gtk_list_store_new (3, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING);
   gtk_entry_completion_set_text_column (completion, COMPLETION_COL_TEXT);
   gtk_entry_completion_set_match_func (completion,
                contact_selector_dialog_match_func,
                NULL, NULL);
-  gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (model));
+  gtk_entry_completion_set_model (completion, GTK_TREE_MODEL (priv->store));
   gtk_entry_set_completion (GTK_ENTRY (priv->entry_id), completion);
   g_signal_connect (completion, "match-selected",
         G_CALLBACK (contact_selector_dialog_match_selected_cb),
         dialog);
   g_object_unref (completion);
-  g_object_unref (model);
+  g_object_unref (priv->store);
 
   empathy_builder_connect (gui, dialog,
              "entry_id", "changed", contact_selector_change_state_button_cb,
@@ -342,6 +380,11 @@ empathy_contact_selector_dialog_get_property (GObject *self,
           empathy_contact_selector_dialog_get_show_account_chooser (dialog));
         break;
 
+      case PROP_FILTER_ACCOUNT:
+        g_value_set_object (value,
+            empathy_contact_selector_dialog_get_filter_account (dialog));
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (self, prop_id, pspec);
         break;
@@ -363,12 +406,38 @@ empathy_contact_selector_dialog_set_property (GObject *self,
             g_value_get_boolean (value));
         break;
 
+      case PROP_FILTER_ACCOUNT:
+        empathy_contact_selector_dialog_set_filter_account (dialog,
+            g_value_get_object (value));
+        break;
+
       default:
         G_OBJECT_WARN_INVALID_PROPERTY_ID (self, prop_id, pspec);
         break;
     }
 }
 
+static void
+empathy_contact_selector_dialog_constructed (GObject *dialog)
+{
+  EmpathyContactSelectorDialogPriv *priv = GET_PRIV (dialog);
+
+  if (EMPATHY_CONTACT_SELECTOR_DIALOG_GET_CLASS (dialog)->contact_filter)
+    {
+      GtkEntryCompletion *completion;
+      GtkTreeModel *filter;
+
+      completion = gtk_entry_get_completion (GTK_ENTRY (priv->entry_id));
+      filter = gtk_tree_model_filter_new (GTK_TREE_MODEL (priv->store), NULL);
+
+      gtk_tree_model_filter_set_visible_func (GTK_TREE_MODEL_FILTER (filter),
+          contact_selector_dialog_filter_visible, dialog, NULL);
+
+      gtk_entry_completion_set_model (completion, filter);
+      g_object_unref (filter);
+    }
+}
+
 static void
 empathy_contact_selector_dialog_dispose (GObject *object)
 {
@@ -379,6 +448,11 @@ empathy_contact_selector_dialog_dispose (GObject *object)
     priv->contact_manager = NULL;
   }
 
+  if (priv->filter_account != NULL) {
+    g_object_unref (priv->filter_account);
+    priv->filter_account = NULL;
+  }
+
   if (G_OBJECT_CLASS (empathy_contact_selector_dialog_parent_class)->dispose)
     G_OBJECT_CLASS (empathy_contact_selector_dialog_parent_class)->dispose (
         object);
@@ -392,6 +466,9 @@ empathy_contact_selector_dialog_class_init (
 
   g_type_class_add_private (class, sizeof (EmpathyContactSelectorDialogPriv));
 
+  class->contact_filter = NULL;
+
+  object_class->constructed = empathy_contact_selector_dialog_constructed;
   object_class->dispose = empathy_contact_selector_dialog_dispose;
   object_class->get_property = empathy_contact_selector_dialog_get_property;
   object_class->set_property = empathy_contact_selector_dialog_set_property;
@@ -402,6 +479,14 @@ empathy_contact_selector_dialog_class_init (
         "Whether or not this dialog should show an account chooser",
         TRUE,
         G_PARAM_READWRITE));
+
+  g_object_class_install_property (object_class, PROP_FILTER_ACCOUNT,
+      g_param_spec_object ("filter-account",
+        "Account to filter contacts",
+        "if 'show-account-chooser' is unset, only the contacts from this "
+        "account are displayed",
+        TP_TYPE_ACCOUNT,
+        G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
 }
 
 const gchar *
@@ -459,3 +544,31 @@ empathy_contact_selector_dialog_get_show_account_chooser (
   priv = GET_PRIV (self);
   return priv->show_account_chooser;
 }
+
+void
+empathy_contact_selector_dialog_set_filter_account (
+    EmpathyContactSelectorDialog *self,
+    TpAccount *account)
+{
+  EmpathyContactSelectorDialogPriv *priv;
+
+  g_return_if_fail (EMPATHY_IS_CONTACT_SELECTOR_DIALOG (self));
+
+  priv = GET_PRIV (self);
+  priv->filter_account = g_object_ref (account);
+
+  g_object_notify (G_OBJECT (self), "filter-account");
+}
+
+TpAccount *
+empathy_contact_selector_dialog_get_filter_account (
+    EmpathyContactSelectorDialog *self)
+{
+  EmpathyContactSelectorDialogPriv *priv;
+
+  g_return_val_if_fail (EMPATHY_IS_CONTACT_SELECTOR_DIALOG (self), NULL);
+
+  priv = GET_PRIV (self);
+  return priv->filter_account;
+
+}
