@@ -31,13 +31,8 @@
 #include <gtk/gtk.h>
 
 #include <telepathy-glib/account-manager.h>
-#ifdef ENABLE_TPL
 #include <telepathy-logger/log-manager.h>
-#endif /* ENABLE_TPL */
 
-#ifndef ENABLE_TPL
-#include <libempathy/empathy-log-manager.h>
-#endif /* ENABLE_TPL */
 #include <libempathy/empathy-chatroom-manager.h>
 #include <libempathy/empathy-chatroom.h>
 #include <libempathy/empathy-message.h>
@@ -76,11 +71,7 @@ typedef struct {
 
 	gchar             *last_find;
 
-#ifndef ENABLE_TPL
-	EmpathyLogManager *log_manager;
-#else
 	TplLogManager     *log_manager;
-#endif /* ENABLE_TPL */
 
 	/* Those are only used while waiting for the account chooser to be ready */
 	TpAccount         *selected_account;
@@ -99,6 +90,8 @@ static void     log_window_find_populate                   (EmpathyLogWindow *wi
 static void     log_window_find_setup                      (EmpathyLogWindow *window);
 static void     log_window_button_find_clicked_cb          (GtkWidget        *widget,
 							    EmpathyLogWindow *window);
+static void     log_window_entry_find_activate_cb          (GtkWidget        *widget,
+							    EmpathyLogWindow *window);
 static void     log_window_button_next_clicked_cb          (GtkWidget        *widget,
 							    EmpathyLogWindow *window);
 static void     log_window_button_previous_clicked_cb      (GtkWidget        *widget,
@@ -118,7 +111,7 @@ static gboolean log_window_chats_get_selected              (EmpathyLogWindow *wi
 							    gchar           **chat_id,
 							    gboolean         *is_chatroom);
 static void     log_window_chats_get_messages              (EmpathyLogWindow *window,
-							    const gchar      *date_to_show);
+							    GDate      *date_to_show);
 static void     log_window_calendar_chats_day_selected_cb  (GtkWidget        *calendar,
 							    EmpathyLogWindow *window);
 static void     log_window_calendar_chats_month_changed_cb (GtkWidget        *calendar,
@@ -149,6 +142,8 @@ enum {
 	COL_CHAT_COUNT
 };
 
+static EmpathyLogWindow *log_window = NULL;
+
 static void
 account_manager_prepared_cb (GObject *source_object,
 			     GAsyncResult *result,
@@ -160,6 +155,9 @@ account_manager_prepared_cb (GObject *source_object,
 	GList *accounts;
 	GError *error = NULL;
 
+	if (log_window == NULL)
+		return;
+
 	if (!tp_account_manager_prepare_finish (account_manager, result, &error)) {
 		DEBUG ("Failed to prepare account manager: %s", error->message);
 		g_error_free (error);
@@ -194,30 +192,28 @@ empathy_log_window_show (TpAccount  *account,
 			gboolean     is_chatroom,
 			GtkWindow   *parent)
 {
-	static EmpathyLogWindow *window = NULL;
 	EmpathyAccountChooser   *account_chooser;
 	TpAccountManager        *account_manager;
 	GtkBuilder             *gui;
 	gchar                  *filename;
+	EmpathyLogWindow       *window;
 
-	if (window) {
-		gtk_window_present (GTK_WINDOW (window->window));
+	if (log_window) {
+		gtk_window_present (GTK_WINDOW (log_window->window));
 
 		if (account && chat_id) {
-			gtk_notebook_set_current_page (GTK_NOTEBOOK (window->notebook), 1);
-			log_window_chats_set_selected (window, account,
+			gtk_notebook_set_current_page (GTK_NOTEBOOK (log_window->notebook), 1);
+			log_window_chats_set_selected (log_window, account,
 						       chat_id, is_chatroom);
 		}
 
-		return window->window;
+		return log_window->window;
 	}
 
-	window = g_new0 (EmpathyLogWindow, 1);
-#ifndef ENABLE_TPL
-	window->log_manager = empathy_log_manager_dup_singleton ();
-#else
-	window->log_manager = tpl_log_manager_dup_singleton ();
-#endif /* ENABLE_TPL */
+	log_window = g_new0 (EmpathyLogWindow, 1);
+	log_window->log_manager = tpl_log_manager_dup_singleton ();
+
+	window = log_window;
 
 	filename = empathy_file_lookup ("empathy-log-window.ui",
 					"libempathy-gtk");
@@ -241,6 +237,7 @@ empathy_log_window_show (TpAccount  *account,
 	empathy_builder_connect (gui, window,
 			      "log_window", "destroy", log_window_destroy_cb,
 			      "entry_find", "changed", log_window_entry_find_changed_cb,
+			      "entry_find", "activate", log_window_entry_find_activate_cb,
 			      "button_previous", "clicked", log_window_button_previous_clicked_cb,
 			      "button_next", "clicked", log_window_button_next_clicked_cb,
 			      "button_find", "clicked", log_window_button_find_clicked_cb,
@@ -251,7 +248,7 @@ empathy_log_window_show (TpAccount  *account,
 	g_object_unref (gui);
 
 	g_object_add_weak_pointer (G_OBJECT (window->window),
-				   (gpointer) &window);
+				   (gpointer) &log_window);
 
 	/* We set this up here so we can block it when needed. */
 	g_signal_connect (window->calendar_chats, "day-selected",
@@ -352,7 +349,6 @@ log_window_entry_find_changed_cb (GtkWidget       *entry,
 	gtk_widget_set_sensitive (window->button_find, is_sensitive);
 }
 
-#ifdef ENABLE_TPL
 static void
 got_messages_for_date_cb (GObject *manager,
                        GAsyncResult *result,
@@ -365,9 +361,11 @@ got_messages_for_date_cb (GObject *manager,
 	gboolean       can_do_next;
 	GError        *error = NULL;
 
-	messages = tpl_log_manager_get_messages_for_date_async_finish (result, &error);
+	if (log_window == NULL)
+		return;
 
-	if (error != NULL) {
+	if (!tpl_log_manager_get_messages_for_date_finish (TPL_LOG_MANAGER (manager),
+		result, &messages, &error)) {
 			DEBUG ("Unable to retrieve messages for the selected date: %s. Aborting",
 					error->message);
 			empathy_chat_view_append_event (window->chatview_find,
@@ -379,7 +377,7 @@ got_messages_for_date_cb (GObject *manager,
 	for (l = messages; l; l = l->next) {
 			EmpathyMessage *message;
 
-			g_assert (TPL_IS_LOG_ENTRY (l->data));
+			g_assert (TPL_IS_ENTRY (l->data));
 
 			message = empathy_message_from_tpl_log_entry (l->data);
 			g_object_unref (l->data);
@@ -408,7 +406,25 @@ got_messages_for_date_cb (GObject *manager,
 	gtk_widget_set_sensitive (window->button_next, can_do_next);
 	gtk_widget_set_sensitive (window->button_find, FALSE);
 }
-#endif /* ENABLE_TPL */
+
+static GDate *
+gdate_from_str (const gchar *str)
+{
+	guint u;
+	guint day, month, year;
+
+	if (sscanf (str, "%u", &u) != 1)
+		return NULL;
+
+	day = (u % 100);
+	month = ((u / 100) % 100);
+	year = (u / 10000);
+
+	if (!g_date_valid_dmy (day, month, year))
+		return NULL;
+
+	return g_date_new_dmy (day, month, year);
+}
 
 static void
 log_window_find_changed_cb (GtkTreeSelection *selection,
@@ -421,13 +437,7 @@ log_window_find_changed_cb (GtkTreeSelection *selection,
 	gchar         *chat_id;
 	gboolean       is_chatroom;
 	gchar         *date;
-#ifndef ENABLE_TPL
-	EmpathyMessage *message;
-	GList         *messages;
-	GList         *l;
-	gboolean       can_do_previous;
-	gboolean       can_do_next;
-#endif /* ENABLE_TPL */
+	GDate         *gdate;
 
 	/* Get selected information */
 	view = GTK_TREE_VIEW (window->treeview_find);
@@ -459,56 +469,26 @@ log_window_find_changed_cb (GtkTreeSelection *selection,
 	empathy_chat_view_scroll (window->chatview_find, FALSE);
 
 	/* Get messages */
-#ifndef ENABLE_TPL
-	messages = empathy_log_manager_get_messages_for_date (window->log_manager,
-							      account,
-							      chat_id,
-							      is_chatroom,
-							      date);
-#else
-	tpl_log_manager_get_messages_for_date_async (window->log_manager,
-							      account,
-							      chat_id,
-							      is_chatroom,
-							      date,
-							      got_messages_for_date_cb,
-							      window);
-#endif /* ENABLE_TPL */
+	gdate = gdate_from_str (date);
+
+	if (gdate != NULL) {
+		tpl_log_manager_get_messages_for_date_async (window->log_manager,
+								      account,
+								      chat_id,
+								      is_chatroom,
+								      gdate,
+								      got_messages_for_date_cb,
+								      window);
+
+		g_date_free (gdate);
+	}
+
 	g_object_unref (account);
 	g_free (date);
 	g_free (chat_id);
-
-#ifndef ENABLE_TPL
-	for (l = messages; l; l = l->next) {
-		message = l->data;
-		empathy_chat_view_append_message (window->chatview_find, message);
-		g_object_unref (message);
-	}
-	g_list_free (messages);
-
-	/* Scroll to the most recent messages */
-	empathy_chat_view_scroll (window->chatview_find, TRUE);
-
-	/* Highlight and find messages */
-	empathy_chat_view_highlight (window->chatview_find,
-				    window->last_find, FALSE);
-	empathy_chat_view_find_next (window->chatview_find,
-				    window->last_find,
-				    TRUE,
-				    FALSE);
-	empathy_chat_view_find_abilities (window->chatview_find,
-					 window->last_find,
-					 FALSE,
-					 &can_do_previous,
-					 &can_do_next);
-	gtk_widget_set_sensitive (window->button_previous, can_do_previous);
-	gtk_widget_set_sensitive (window->button_next, can_do_next);
-	gtk_widget_set_sensitive (window->button_find, FALSE);
-#endif /* ENABLE_TPL */
 }
 
 
-#ifdef ENABLE_TPL
 static void
 log_manager_searched_new_cb (GObject *manager,
                              GAsyncResult *result,
@@ -520,9 +500,11 @@ log_manager_searched_new_cb (GObject *manager,
 	GtkListStore        *store = user_data;
 	GError              *error = NULL;
 
-	hits = tpl_log_manager_search_new_async_finish (result, &error);
+	if (log_window == NULL)
+		return;
 
-	if (error != NULL) {
+	if (!tpl_log_manager_search_finish (TPL_LOG_MANAGER (manager), result,
+		&hits, &error)) {
 			DEBUG ("%s. Aborting", error->message);
 			g_error_free (error);
 			return;
@@ -532,7 +514,8 @@ log_manager_searched_new_cb (GObject *manager,
 			TplLogSearchHit *hit;
 			const gchar         *account_name;
 			const gchar         *account_icon;
-			gchar               *date_readable;
+			gchar               date_readable[255];
+			gchar               tmp[255];
 
 			hit = l->data;
 
@@ -541,7 +524,12 @@ log_manager_searched_new_cb (GObject *manager,
 					continue;
 			}
 
-			date_readable = tpl_log_manager_get_date_readable (hit->date);
+			g_date_strftime (date_readable, sizeof(date_readable),
+				EMPATHY_TIME_FORMAT_DISPLAY_LONG, hit->date);
+
+			g_date_strftime (tmp, sizeof(tmp),
+				"%Y%m%d", hit->date);
+
 			account_name = tp_account_get_display_name (hit->account);
 			account_icon = tp_account_get_icon_name (hit->account);
 
@@ -553,12 +541,10 @@ log_manager_searched_new_cb (GObject *manager,
 					COL_FIND_CHAT_NAME, hit->chat_id, /* FIXME */
 					COL_FIND_CHAT_ID, hit->chat_id,
 					COL_FIND_IS_CHATROOM, hit->is_chatroom,
-					COL_FIND_DATE, hit->date,
+					COL_FIND_DATE, tmp,
 					COL_FIND_DATE_READABLE, date_readable,
 					-1);
 
-			g_free (date_readable);
-
 			/* FIXME: Update COL_FIND_CHAT_NAME */
 			if (hit->is_chatroom) {
 			} else {
@@ -569,24 +555,15 @@ log_manager_searched_new_cb (GObject *manager,
 			tpl_log_manager_search_free (hits);
 	}
 }
-#endif /* ENABLE_TPL */
-
 
 static void
 log_window_find_populate (EmpathyLogWindow *window,
 			  const gchar     *search_criteria)
 {
-#ifndef ENABLE_TPL
-	GList              *hits, *l;
-
-#endif /* ENABLE_TPL */
 	GtkTreeView        *view;
 	GtkTreeModel       *model;
 	GtkTreeSelection   *selection;
 	GtkListStore       *store;
-#ifndef ENABLE_TPL
-	GtkTreeIter         iter;
-#endif /* ENABLE_TPL */
 
 	view = GTK_TREE_VIEW (window->treeview_find);
 	model = gtk_tree_view_get_model (view);
@@ -602,53 +579,8 @@ log_window_find_populate (EmpathyLogWindow *window,
 		return;
 	}
 
-#ifdef ENABLE_TPL
-	tpl_log_manager_search_new_async (window->log_manager, search_criteria,
+	tpl_log_manager_search_async (window->log_manager, search_criteria,
 			log_manager_searched_new_cb, (gpointer) store);
-#else
-	hits = empathy_log_manager_search_new (window->log_manager, search_criteria);
-
-	for (l = hits; l; l = l->next) {
-		EmpathyLogSearchHit *hit;
-		const gchar         *account_name;
-		const gchar         *account_icon;
-		gchar               *date_readable;
-
-		hit = l->data;
-
-		/* Protect against invalid data (corrupt or old log files. */
-		if (!hit->account || !hit->chat_id) {
-			continue;
-		}
-
-		date_readable = empathy_log_manager_get_date_readable (hit->date);
-		account_name = tp_account_get_display_name (hit->account);
-		account_icon = tp_account_get_icon_name (hit->account);
-
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter,
-				    COL_FIND_ACCOUNT_ICON, account_icon,
-				    COL_FIND_ACCOUNT_NAME, account_name,
-				    COL_FIND_ACCOUNT, hit->account,
-				    COL_FIND_CHAT_NAME, hit->chat_id, /* FIXME */
-				    COL_FIND_CHAT_ID, hit->chat_id,
-				    COL_FIND_IS_CHATROOM, hit->is_chatroom,
-				    COL_FIND_DATE, hit->date,
-				    COL_FIND_DATE_READABLE, date_readable,
-				    -1);
-
-		g_free (date_readable);
-
-		/* FIXME: Update COL_FIND_CHAT_NAME */
-		if (hit->is_chatroom) {
-		} else {
-		}
-	}
-
-	if (hits) {
-		empathy_log_manager_search_free (hits);
-	}
-#endif /* ENABLE_TPL */
 }
 
 static void
@@ -738,8 +670,7 @@ log_window_find_setup (EmpathyLogWindow *window)
 }
 
 static void
-log_window_button_find_clicked_cb (GtkWidget       *widget,
-				   EmpathyLogWindow *window)
+start_find_search (EmpathyLogWindow *window)
 {
 	const gchar *str;
 
@@ -756,6 +687,20 @@ log_window_button_find_clicked_cb (GtkWidget       *widget,
 	log_window_find_populate (window, str);
 }
 
+static void
+log_window_button_find_clicked_cb (GtkWidget       *widget,
+				   EmpathyLogWindow *window)
+{
+	start_find_search (window);
+}
+
+static void
+log_window_entry_find_activate_cb (GtkWidget *entry,
+				   EmpathyLogWindow *self)
+{
+	start_find_search (self);
+}
+
 static void
 log_window_button_next_clicked_cb (GtkWidget       *widget,
 				   EmpathyLogWindow *window)
@@ -814,7 +759,6 @@ log_window_chats_changed_cb (GtkTreeSelection *selection,
 	log_window_chats_get_messages (window, NULL);
 }
 
-#ifdef ENABLE_TPL
 static void
 log_manager_got_chats_cb (GObject *manager,
                        GAsyncResult *result,
@@ -832,9 +776,11 @@ log_manager_got_chats_cb (GObject *manager,
 	GtkTreeIter            iter;
 	GError                *error = NULL;
 
-	chats = tpl_log_manager_get_chats_async_finish (result, &error);
+	if (log_window == NULL)
+		return;
 
-	if (error != NULL) {
+	if (!tpl_log_manager_get_chats_finish (TPL_LOG_MANAGER (manager),
+		result, &chats, &error)) {
 			DEBUG ("%s. Aborting", error->message);
 			g_error_free (error);
 			return;
@@ -876,25 +822,17 @@ log_manager_got_chats_cb (GObject *manager,
 
 	g_object_unref (account);
 }
-#endif /* ENABLE_TPL */
-
 
 static void
 log_window_chats_populate (EmpathyLogWindow *window)
 {
 	EmpathyAccountChooser *account_chooser;
 	TpAccount             *account;
-#ifndef ENABLE_TPL
-	GList                *chats, *l;
-#endif /* ENABLE_TPL */
 
 	GtkTreeView          *view;
 	GtkTreeModel         *model;
 	GtkTreeSelection     *selection;
 	GtkListStore         *store;
-#ifndef ENABLE_TPL
-	GtkTreeIter           iter;
-#endif /* ENABLE_TPL */
 
 	account_chooser = EMPATHY_ACCOUNT_CHOOSER (window->account_chooser_chats);
 	account = empathy_account_chooser_dup_account (account_chooser);
@@ -916,40 +854,8 @@ log_window_chats_populate (EmpathyLogWindow *window)
 
 	gtk_list_store_clear (store);
 
-#ifdef ENABLE_TPL
 	tpl_log_manager_get_chats_async (window->log_manager, account,
 			log_manager_got_chats_cb, (gpointer) window);
-#else
-	chats = empathy_log_manager_get_chats (window->log_manager, account);
-	for (l = chats; l; l = l->next) {
-		EmpathyLogSearchHit *hit;
-
-		hit = l->data;
-
-		gtk_list_store_append (store, &iter);
-		gtk_list_store_set (store, &iter,
-				    COL_CHAT_ICON, "empathy-available", /* FIXME */
-				    COL_CHAT_NAME, hit->chat_id,
-				    COL_CHAT_ACCOUNT, account,
-				    COL_CHAT_ID, hit->chat_id,
-				    COL_CHAT_IS_CHATROOM, hit->is_chatroom,
-				    -1);
-
-		/* FIXME: Update COL_CHAT_ICON/NAME */
-		if (hit->is_chatroom) {
-		} else {
-		}
-	}
-	empathy_log_manager_search_free (chats);
-
-	/* Unblock signals */
-	g_signal_handlers_unblock_by_func (selection,
-					   log_window_chats_changed_cb,
-					   window);
-
-
-	g_object_unref (account);
-#endif /* ENABLE_TPL */
 }
 
 static void
@@ -1119,7 +1025,6 @@ log_window_chats_get_selected (EmpathyLogWindow  *window,
 	return TRUE;
 }
 
-#ifdef ENABLE_TPL
 static void
 log_window_got_messages_for_date_cb (GObject *manager,
     GAsyncResult *result,
@@ -1130,9 +1035,11 @@ log_window_got_messages_for_date_cb (GObject *manager,
   GList *l;
   GError *error = NULL;
 
-  messages = tpl_log_manager_get_messages_for_date_async_finish (result, &error);
+  if (log_window == NULL)
+    return;
 
-  if (error != NULL) {
+  if (!tpl_log_manager_get_messages_for_date_finish (TPL_LOG_MANAGER (manager),
+        result, &messages, &error)) {
       DEBUG ("Unable to retrieve messages for the selected date: %s. Aborting",
           error->message);
       empathy_chat_view_append_event (window->chatview_find,
@@ -1160,14 +1067,12 @@ log_window_got_messages_for_date_cb (GObject *manager,
 
 static void
 log_window_get_messages_for_date (EmpathyLogWindow *window,
-                                 const gchar *date)
+				  GDate *date)
 {
   TpAccount *account;
   gchar *chat_id;
   gboolean is_chatroom;
 
-  gtk_calendar_clear_marks (GTK_CALENDAR (window->calendar_chats));
-
   if (!log_window_chats_get_selected (window, &account,
         &chat_id, &is_chatroom)) {
       return;
@@ -1197,17 +1102,16 @@ log_manager_got_dates_cb (GObject *manager,
   GList         *dates;
   GList         *l;
   guint          year_selected;
-  guint          year;
-  guint          month;
   guint          month_selected;
-  guint          day;
   gboolean       day_selected = FALSE;
-  const gchar   *date = NULL;
+  GDate         *date = NULL;
   GError        *error = NULL;
 
-  dates = tpl_log_manager_get_dates_async_finish (result, &error);
+  if (log_window == NULL)
+    return;
 
-  if (error != NULL) {
+  if (!tpl_log_manager_get_dates_finish (TPL_LOG_MANAGER (manager),
+        result, &dates, &error)) {
     DEBUG ("Unable to retrieve messages' dates: %s. Aborting",
         error->message);
     empathy_chat_view_append_event (window->chatview_find,
@@ -1216,14 +1120,8 @@ log_manager_got_dates_cb (GObject *manager,
   }
 
   for (l = dates; l; l = l->next) {
-      const gchar *str;
+      GDate *d = l->data;
 
-      str = l->data;
-      if (!str) {
-          continue;
-      }
-
-      sscanf (str, "%4d%2d%2d", &year, &month, &day);
       gtk_calendar_get_date (GTK_CALENDAR (window->calendar_chats),
           &year_selected,
           &month_selected,
@@ -1232,15 +1130,19 @@ log_manager_got_dates_cb (GObject *manager,
       month_selected++;
 
       if (!l->next) {
-          date = str;
+          date = d;
       }
 
-      if (year != year_selected || month != month_selected) {
+      if (g_date_get_year (d) != year_selected ||
+          g_date_get_month (d) != month_selected) {
           continue;
       }
 
-      DEBUG ("Marking date:'%s'", str);
-      gtk_calendar_mark_day (GTK_CALENDAR (window->calendar_chats), day);
+      DEBUG ("Marking date: %04u-%02u-%02u", g_date_get_year (d),
+          g_date_get_month (d), g_date_get_day (d));
+
+      gtk_calendar_mark_day (GTK_CALENDAR (window->calendar_chats),
+          g_date_get_day (d));
 
       if (l->next) {
           continue;
@@ -1248,7 +1150,8 @@ log_manager_got_dates_cb (GObject *manager,
 
       day_selected = TRUE;
 
-      gtk_calendar_select_day (GTK_CALENDAR (window->calendar_chats), day);
+      gtk_calendar_select_day (GTK_CALENDAR (window->calendar_chats),
+          g_date_get_day (d));
   }
 
   if (!day_selected) {
@@ -1261,6 +1164,7 @@ log_manager_got_dates_cb (GObject *manager,
       window);
 
   if (date) {
+      /* Show messages of the most recent date */
       log_window_get_messages_for_date (window, date);
   }
 
@@ -1271,15 +1175,12 @@ log_manager_got_dates_cb (GObject *manager,
 
 static void
 log_window_chats_get_messages (EmpathyLogWindow *window,
-			       const gchar     *date_to_show)
+			       GDate     *date)
 {
 	TpAccount     *account;
 	gchar         *chat_id;
 	gboolean       is_chatroom;
-	const gchar   *date;
 	guint          year_selected;
-	guint          year;
-	guint          month;
 	guint          month_selected;
 	guint          day;
 
@@ -1294,8 +1195,7 @@ log_window_chats_get_messages (EmpathyLogWindow *window,
 					 window);
 
 	/* Either use the supplied date or get the last */
-	date = date_to_show;
-	if (!date) {
+	if (date == NULL) {
 		/* Get a list of dates and show them on the calendar */
 		tpl_log_manager_get_dates_async (window->log_manager,
 						       account, chat_id,
@@ -1303,7 +1203,7 @@ log_window_chats_get_messages (EmpathyLogWindow *window,
 						       log_manager_got_dates_cb, (gpointer) window);
     /* signal unblocked at the end of the CB flow */
 	} else {
-		sscanf (date, "%4d%2d%2d", &year, &month, &day);
+		day = g_date_get_day (date);
 		gtk_calendar_get_date (GTK_CALENDAR (window->calendar_chats),
 				&year_selected,
 				&month_selected,
@@ -1311,7 +1211,8 @@ log_window_chats_get_messages (EmpathyLogWindow *window,
 
 		month_selected++;
 
-		if (year != year_selected && month != month_selected) {
+		if (g_date_get_year (date) != year_selected &&
+			g_date_get_month (date) != month_selected) {
 			day = 0;
 		}
 
@@ -1322,154 +1223,15 @@ log_window_chats_get_messages (EmpathyLogWindow *window,
         window);
 	}
 
-	if (date) {
-      log_window_get_messages_for_date (window, date);
+	if (date != NULL) {
+		/* Show messages of the selected date */
+		log_window_get_messages_for_date (window, date);
 	}
 
 	g_object_unref (account);
 	g_free (chat_id);
 }
 
-#else
-
-static void
-log_window_chats_get_messages (EmpathyLogWindow *window,
-			       const gchar     *date_to_show)
-{
-	TpAccount     *account;
-	gchar         *chat_id;
-	gboolean       is_chatroom;
-	EmpathyMessage *message;
-	GList         *messages;
-	GList         *dates = NULL;
-	GList         *l;
-	const gchar   *date;
-	guint          year_selected;
-	guint          year;
-	guint          month;
-	guint          month_selected;
-	guint          day;
-
-	if (!log_window_chats_get_selected (window, &account,
-					    &chat_id, &is_chatroom)) {
-		return;
-	}
-
-	g_signal_handlers_block_by_func (window->calendar_chats,
-					 log_window_calendar_chats_day_selected_cb,
-					 window);
-
-	/* Either use the supplied date or get the last */
-	date = date_to_show;
-	if (!date) {
-		gboolean day_selected = FALSE;
-
-		/* Get a list of dates and show them on the calendar */
-		dates = empathy_log_manager_get_dates (window->log_manager,
-						       account, chat_id,
-						       is_chatroom);
-
-		for (l = dates; l; l = l->next) {
-			const gchar *str;
-
-			str = l->data;
-			if (!str) {
-				continue;
-			}
-
-			sscanf (str, "%4d%2d%2d", &year, &month, &day);
-			gtk_calendar_get_date (GTK_CALENDAR (window->calendar_chats),
-					       &year_selected,
-					       &month_selected,
-					       NULL);
-
-			month_selected++;
-
-			if (!l->next) {
-				date = str;
-			}
-
-			if (year != year_selected || month != month_selected) {
-				continue;
-			}
-
-
-			DEBUG ("Marking date:'%s'", str);
-			gtk_calendar_mark_day (GTK_CALENDAR (window->calendar_chats), day);
-
-			if (l->next) {
-				continue;
-			}
-
-			day_selected = TRUE;
-
-			gtk_calendar_select_day (GTK_CALENDAR (window->calendar_chats), day);
-		}
-
-		if (!day_selected) {
-			/* Unselect the day in the calendar */
-			gtk_calendar_select_day (GTK_CALENDAR (window->calendar_chats), 0);
-		}
-	} else {
-		sscanf (date, "%4d%2d%2d", &year, &month, &day);
-		gtk_calendar_get_date (GTK_CALENDAR (window->calendar_chats),
-				       &year_selected,
-				       &month_selected,
-				       NULL);
-
-		month_selected++;
-
-		if (year != year_selected && month != month_selected) {
-			day = 0;
-		}
-
-		gtk_calendar_select_day (GTK_CALENDAR (window->calendar_chats), day);
-	}
-
-	g_signal_handlers_unblock_by_func (window->calendar_chats,
-					   log_window_calendar_chats_day_selected_cb,
-					   window);
-
-	if (!date) {
-		goto OUT;
-	}
-
-	/* Clear all current messages shown in the textview */
-	empathy_chat_view_clear (window->chatview_chats);
-
-	/* Turn off scrolling temporarily */
-	empathy_chat_view_scroll (window->chatview_find, FALSE);
-
-	/* Get messages */
-	messages = empathy_log_manager_get_messages_for_date (window->log_manager,
-							      account, chat_id,
-							      is_chatroom,
-							      date);
-
-	for (l = messages; l; l = l->next) {
-		message = l->data;
-
-		empathy_chat_view_append_message (window->chatview_chats,
-						 message);
-		g_object_unref (message);
-	}
-	g_list_free (messages);
-
-	/* Turn back on scrolling */
-	empathy_chat_view_scroll (window->chatview_find, TRUE);
-
-	/* Give the search entry main focus */
-	gtk_widget_grab_focus (window->entry_chats);
-
-OUT:
-	g_list_foreach (dates, (GFunc) g_free, NULL);
-	g_list_free (dates);
-	g_object_unref (account);
-	g_free (chat_id);
-}
-
-#endif /* ENABLE_TPL */
-
 static void
 log_window_calendar_chats_day_selected_cb (GtkWidget       *calendar,
 					   EmpathyLogWindow *window)
@@ -1477,25 +1239,25 @@ log_window_calendar_chats_day_selected_cb (GtkWidget       *calendar,
 	guint  year;
 	guint  month;
 	guint  day;
-
-	gchar *date;
+	GDate *date;
 
 	gtk_calendar_get_date (GTK_CALENDAR (calendar), &year, &month, &day);
+	if (day == 0)
+		/* No date selected */
+		return;
 
 	/* We need this hear because it appears that the months start from 0 */
 	month++;
 
-	date = g_strdup_printf ("%4.4d%2.2d%2.2d", year, month, day);
+	date = g_date_new_dmy (day, month, year);
 
-	DEBUG ("Currently selected date is:'%s'", date);
+	DEBUG ("Currently selected date is: %04u-%02u-%02u", year, month, day);
 
 	log_window_chats_get_messages (window, date);
 
-	g_free (date);
+	g_date_free (date);
 }
 
-
-#ifdef ENABLE_TPL
 static void
 log_window_updating_calendar_month_cb (GObject *manager,
 		GAsyncResult *result, gpointer user_data)
@@ -1507,9 +1269,11 @@ log_window_updating_calendar_month_cb (GObject *manager,
 	guint					 month_selected;
 	GError				*error = NULL;
 
-	dates = tpl_log_manager_get_dates_async_finish (result, &error);
+	if (log_window == NULL)
+		return;
 
-	if (error != NULL) {
+	if (!tpl_log_manager_get_dates_finish (TPL_LOG_MANAGER (manager),
+		result, &dates, &error)) {
 			DEBUG ("Unable to retrieve messages' dates: %s. Aborting",
 					error->message);
 			empathy_chat_view_append_event (window->chatview_find,
@@ -1528,21 +1292,13 @@ log_window_updating_calendar_month_cb (GObject *manager,
 	month_selected++;
 
 	for (l = dates; l; l = l->next) {
-			const gchar *str;
-			guint        year;
-			guint        month;
-			guint        day;
+			GDate *date = l->data;
 
-			str = l->data;
-			if (!str) {
-					continue;
-			}
-
-			sscanf (str, "%4d%2d%2d", &year, &month, &day);
-
-			if (year == year_selected && month == month_selected) {
-					DEBUG ("Marking date:'%s'", str);
-					gtk_calendar_mark_day (GTK_CALENDAR (window->calendar_chats), day);
+			if (g_date_get_year (date) == year_selected &&
+			    g_date_get_month (date) == month_selected) {
+					DEBUG ("Marking date: %04u-%02u-%02u", g_date_get_year (date),
+						g_date_get_month (date), g_date_get_day (date));
+					gtk_calendar_mark_day (GTK_CALENDAR (window->calendar_chats), g_date_get_day (date));
 			}
 	}
 
@@ -1552,8 +1308,6 @@ log_window_updating_calendar_month_cb (GObject *manager,
 	DEBUG ("Currently showing month %d and year %d", month_selected,
 			year_selected);
 }
-#endif /* ENABLE_TPL */
-
 
 static void
 log_window_calendar_chats_month_changed_cb (GtkWidget       *calendar,
@@ -1562,13 +1316,6 @@ log_window_calendar_chats_month_changed_cb (GtkWidget       *calendar,
 	TpAccount     *account;
 	gchar         *chat_id;
 	gboolean       is_chatroom;
-#ifndef ENABLE_TPL
-	guint          year_selected;
-	guint          month_selected;
-
-	GList         *dates;
-	GList         *l;
-#endif /* ENABLE_TPL */
 
 	gtk_calendar_clear_marks (GTK_CALENDAR (calendar));
 
@@ -1579,48 +1326,10 @@ log_window_calendar_chats_month_changed_cb (GtkWidget       *calendar,
 	}
 
 	/* Get the log object for this contact */
-#ifdef ENABLE_TPL
 	tpl_log_manager_get_dates_async (window->log_manager, account,
 					       chat_id, is_chatroom,
 					       log_window_updating_calendar_month_cb,
 					       (gpointer) window);
-#else
-	dates = empathy_log_manager_get_dates (window->log_manager, account,
-					       chat_id, is_chatroom);
-
-	g_object_get (calendar,
-		      "month", &month_selected,
-		      "year", &year_selected,
-		      NULL);
-
-	/* We need this here because it appears that the months start from 0 */
-	month_selected++;
-
-	for (l = dates; l; l = l->next) {
-		const gchar *str;
-		guint        year;
-		guint        month;
-		guint        day;
-
-		str = l->data;
-		if (!str) {
-			continue;
-		}
-
-		sscanf (str, "%4d%2d%2d", &year, &month, &day);
-
-		if (year == year_selected && month == month_selected) {
-			DEBUG ("Marking date:'%s'", str);
-			gtk_calendar_mark_day (GTK_CALENDAR (window->calendar_chats), day);
-		}
-	}
-
-	g_list_foreach (dates, (GFunc) g_free, NULL);
-	g_list_free (dates);
-
-	DEBUG ("Currently showing month %d and year %d", month_selected,
-		year_selected);
-#endif /* ENABLE_TPL */
 
 	g_object_unref (account);
 	g_free (chat_id);
@@ -1658,4 +1367,3 @@ log_window_entry_chats_activate_cb (GtkWidget       *entry,
 					    FALSE);
 	}
 }
-
