@@ -28,9 +28,6 @@
 #include <telepathy-glib/account-manager.h>
 #include <telepathy-glib/interfaces.h>
 #include <telepathy-glib/util.h>
-#ifdef ENABEL_TPL
-#include <telepathy-logger/contact.h>
-#endif /* ENABLE_TPL */
 
 #if HAVE_GEOCLUE
 #include <geoclue/geoclue-geocode.h>
@@ -479,32 +476,30 @@ empathy_contact_new (TpContact *tp_contact)
       NULL);
 }
 
-#ifdef ENABLE_TPL
 EmpathyContact *
 empathy_contact_from_tpl_contact (TpAccount *account,
-    TplContact *tpl_contact)
+    TplEntity *tpl_entity)
 {
   EmpathyContact *retval;
   gboolean is_user;
 
-  g_return_val_if_fail (TPL_IS_CONTACT (tpl_contact), NULL);
+  g_return_val_if_fail (TPL_IS_ENTITY (tpl_entity), NULL);
 
-  is_user = (TPL_CONTACT_USER == tpl_contact_get_contact_type (tpl_contact));
+  is_user = (TPL_ENTITY_SELF == tpl_entity_get_entity_type (tpl_entity));
 
   retval = g_object_new (EMPATHY_TYPE_CONTACT,
-      "id", tpl_contact_get_alias (tpl_contact),
-      "name", tpl_contact_get_identifier (tpl_contact),
+      "id", tpl_entity_get_alias (tpl_entity),
+      "name", tpl_entity_get_identifier (tpl_entity),
       "account", account,
       "is-user", is_user,
       NULL);
 
-  if (!EMP_STR_EMPTY (tpl_contact_get_avatar_token (tpl_contact)))
+  if (!EMP_STR_EMPTY (tpl_entity_get_avatar_token (tpl_entity)))
     empathy_contact_load_avatar_cache (retval,
-        tpl_contact_get_avatar_token (tpl_contact));
+        tpl_entity_get_avatar_token (tpl_entity));
 
   return retval;
 }
-#endif /* ENABLE_TPL */
 
 EmpathyContact *
 empathy_contact_new_for_log (TpAccount *account,
@@ -975,7 +970,7 @@ empathy_contact_can_send_files (EmpathyContact *contact)
 }
 
 gboolean
-empathy_contact_can_use_stream_tube (EmpathyContact *contact)
+empathy_contact_can_use_rfb_stream_tube (EmpathyContact *contact)
 {
   EmpathyContactPriv *priv;
 
@@ -983,7 +978,7 @@ empathy_contact_can_use_stream_tube (EmpathyContact *contact)
 
   priv = GET_PRIV (contact);
 
-  return priv->capabilities & EMPATHY_CAPABILITIES_STREAM_TUBE;
+  return priv->capabilities & EMPATHY_CAPABILITIES_RFB_STREAM_TUBE;
 }
 
 static gchar *
@@ -1453,7 +1448,13 @@ tp_caps_to_capabilities (TpCapabilities *caps)
         }
       else if (!tp_strdiff (chan_type, TP_IFACE_CHANNEL_TYPE_STREAM_TUBE))
         {
-          capabilities |= EMPATHY_CAPABILITIES_STREAM_TUBE;
+          const gchar *service;
+
+          service = tp_asv_get_string (fixed_prop,
+              TP_PROP_CHANNEL_TYPE_STREAM_TUBE_SERVICE);
+
+          if (!tp_strdiff (service, "rfb"))
+            capabilities |= EMPATHY_CAPABILITIES_RFB_STREAM_TUBE;
         }
       else if (!tp_strdiff (chan_type,
         TP_IFACE_CHANNEL_TYPE_STREAMED_MEDIA))
