@@ -29,7 +29,6 @@
 
 #include "empathy-tp-chat.h"
 #include "empathy-tp-contact-factory.h"
-#include "empathy-contact-monitor.h"
 #include "empathy-contact-list.h"
 #include "empathy-dispatcher.h"
 #include "empathy-marshal.h"
@@ -42,8 +41,7 @@
 #define GET_PRIV(obj) EMPATHY_GET_PRIV (obj, EmpathyTpChat)
 typedef struct {
 	gboolean               dispose_has_run;
-	EmpathyTpContactFactory *factory;
-	EmpathyContactMonitor *contact_monitor;
+	TpConnection          *connection;
 	EmpathyContact        *user;
 	EmpathyContact        *remote_contact;
 	GList                 *members;
@@ -132,14 +130,12 @@ tp_chat_add (EmpathyContactList *list,
 			-1, &handles, NULL, NULL, NULL, NULL, NULL);
 	} else if (priv->can_upgrade_to_muc) {
 		EmpathyDispatcher *dispatcher;
-		TpConnection      *connection;
 		GHashTable        *props;
 		const char        *object_path;
 		GPtrArray          channels = { (gpointer *) &object_path, 1 };
 		const char        *invitees[2] = { NULL, };
 
 		dispatcher = empathy_dispatcher_dup_singleton ();
-		connection = tp_channel_borrow_connection (priv->channel);
 
 		invitees[0] = empathy_contact_get_id (contact);
 		object_path = tp_proxy_get_object_path (priv->channel);
@@ -159,7 +155,7 @@ tp_chat_add (EmpathyContactList *list,
 		/* Although this is a MUC, it's anonymous, so CreateChannel is
 		 * valid.
 		 * props now belongs to EmpathyDispatcher, don't free it */
-		empathy_dispatcher_create_channel (dispatcher, connection,
+		empathy_dispatcher_create_channel (dispatcher, priv->connection,
 				props, EMPATHY_DISPATCHER_NON_USER_ACTION, NULL, NULL);
 
 		g_object_unref (dispatcher);
@@ -207,20 +203,21 @@ tp_chat_get_members (EmpathyContactList *list)
 	return members;
 }
 
-static EmpathyContactMonitor *
-tp_chat_get_monitor (EmpathyContactList *list)
+static void
+check_ready (EmpathyTpChat *chat)
 {
-	EmpathyTpChatPriv *priv;
+	EmpathyTpChatPriv *priv = GET_PRIV (chat);
 
-	g_return_val_if_fail (EMPATHY_IS_TP_CHAT (list), NULL);
+	if (priv->ready)
+		return;
 
-	priv = GET_PRIV (list);
+	if (g_queue_get_length (priv->messages_queue) > 0)
+		return;
 
-	if (priv->contact_monitor == NULL) {
-		priv->contact_monitor = empathy_contact_monitor_new_for_iface (list);
-	}
+	DEBUG ("Ready");
 
-	return priv->contact_monitor;
+	priv->ready = TRUE;
+	g_object_notify (G_OBJECT (chat), "ready");
 }
 
 static void
@@ -240,10 +237,12 @@ tp_chat_emit_queued_messages (EmpathyTpChat *chat)
 		g_queue_push_tail (priv->pending_messages_queue, message);
 		g_signal_emit (chat, signals[MESSAGE_RECEIVED], 0, message);
 	}
+
+	check_ready (chat);
 }
 
 static void
-tp_chat_got_sender_cb (EmpathyTpContactFactory *factory,
+tp_chat_got_sender_cb (TpConnection            *connection,
 		       EmpathyContact          *contact,
 		       const GError            *error,
 		       gpointer                 message,
@@ -291,7 +290,7 @@ tp_chat_build_message (EmpathyTpChat *chat,
 		empathy_message_set_sender (message, priv->user);
 		tp_chat_emit_queued_messages (chat);
 	} else {
-		empathy_tp_contact_factory_get_from_handle (priv->factory,
+		empathy_tp_contact_factory_get_from_handle (priv->connection,
 			from_handle,
 			tp_chat_got_sender_cb,
 			message, NULL, G_OBJECT (chat));
@@ -412,7 +411,7 @@ typedef struct {
 } StateChangedData;
 
 static void
-tp_chat_state_changed_got_contact_cb (EmpathyTpContactFactory *factory,
+tp_chat_state_changed_got_contact_cb (TpConnection            *connection,
 				      EmpathyContact          *contact,
 				      const GError            *error,
 				      gpointer                 user_data,
@@ -442,7 +441,7 @@ tp_chat_state_changed_cb (TpChannel *channel,
 {
 	EmpathyTpChatPriv *priv = GET_PRIV (chat);
 
-	empathy_tp_contact_factory_get_from_handle (priv->factory, handle,
+	empathy_tp_contact_factory_get_from_handle (priv->connection, handle,
 		tp_chat_state_changed_got_contact_cb, GUINT_TO_POINTER (state),
 		NULL, chat);
 }
@@ -515,6 +514,8 @@ tp_chat_list_pending_messages_cb (TpChannel       *channel,
 		acknowledge_messages (chat, empty_non_text_content_ids);
 		g_array_free (empty_non_text_content_ids, TRUE);
 	}
+
+	check_ready (chat);
 }
 
 static void
@@ -758,6 +759,10 @@ tp_chat_dispose (GObject *object)
 
 	priv->dispose_has_run = TRUE;
 
+	if (priv->connection != NULL)
+		g_object_unref (priv->connection);
+	priv->connection = NULL;
+
 	if (priv->channel != NULL) {
 		g_signal_handlers_disconnect_by_func (priv->channel,
 			tp_chat_invalidated_cb, self);
@@ -769,18 +774,10 @@ tp_chat_dispose (GObject *object)
 		g_object_unref (priv->remote_contact);
 	priv->remote_contact = NULL;
 
-	if (priv->factory != NULL)
-		g_object_unref (priv->factory);
-	priv->factory = NULL;
-
 	if (priv->user != NULL)
 		g_object_unref (priv->user);
 	priv->user = NULL;
 
-	if (priv->contact_monitor)
-		g_object_unref (priv->contact_monitor);
-	priv->contact_monitor = NULL;
-
 	g_queue_foreach (priv->messages_queue, (GFunc) g_object_unref, NULL);
 	g_queue_clear (priv->messages_queue);
 
@@ -821,7 +818,7 @@ tp_chat_finalize (GObject *object)
 }
 
 static void
-tp_chat_check_if_ready (EmpathyTpChat *chat)
+check_almost_ready (EmpathyTpChat *chat)
 {
 	EmpathyTpChatPriv *priv = GET_PRIV (chat);
 
@@ -841,13 +838,14 @@ tp_chat_check_if_ready (EmpathyTpChat *chat)
 	    priv->remote_contact == NULL)
 		return;
 
-	DEBUG ("Ready!");
-
 	tp_cli_channel_type_text_connect_to_received (priv->channel,
 						      tp_chat_received_cb,
 						      NULL, NULL,
 						      G_OBJECT (chat), NULL);
 	priv->listing_pending_messages = TRUE;
+
+	/* TpChat will be ready once ListPendingMessages returned and all the messages
+	 * have been added to the pending messages queue. */
 	tp_cli_channel_type_text_call_list_pending_messages (priv->channel, -1,
 							     FALSE,
 							     tp_chat_list_pending_messages_cb,
@@ -866,8 +864,6 @@ tp_chat_check_if_ready (EmpathyTpChat *chat)
 									   tp_chat_state_changed_cb,
 									   NULL, NULL,
 									   G_OBJECT (chat), NULL);
-	priv->ready = TRUE;
-	g_object_notify (G_OBJECT (chat), "ready");
 }
 
 static void
@@ -931,7 +927,7 @@ tp_chat_update_remote_contact (EmpathyTpChat *chat)
 }
 
 static void
-tp_chat_got_added_contacts_cb (EmpathyTpContactFactory *factory,
+tp_chat_got_added_contacts_cb (TpConnection            *connection,
 			       guint                    n_contacts,
 			       EmpathyContact * const * contacts,
 			       guint                    n_failed,
@@ -966,7 +962,7 @@ tp_chat_got_added_contacts_cb (EmpathyTpContactFactory *factory,
 	}
 
 	tp_chat_update_remote_contact (EMPATHY_TP_CHAT (chat));
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
+	check_almost_ready (EMPATHY_TP_CHAT (chat));
 }
 
 static EmpathyContact *
@@ -1025,7 +1021,7 @@ contact_rename_data_free (ContactRenameData* data)
 }
 
 static void
-tp_chat_got_renamed_contacts_cb (EmpathyTpContactFactory *factory,
+tp_chat_got_renamed_contacts_cb (TpConnection            *connection,
                                  guint                    n_contacts,
                                  EmpathyContact * const * contacts,
                                  guint                    n_failed,
@@ -1076,7 +1072,7 @@ tp_chat_got_renamed_contacts_cb (EmpathyTpContactFactory *factory,
 	}
 
 	tp_chat_update_remote_contact (EMPATHY_TP_CHAT (chat));
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
+	check_almost_ready (EMPATHY_TP_CHAT (chat));
 }
 
 
@@ -1107,7 +1103,7 @@ tp_chat_group_members_changed_cb (TpChannel     *self,
 		old_handle = g_array_index (removed, guint, 0);
 
 		rename_data = contact_rename_data_new (old_handle, reason, message);
-		empathy_tp_contact_factory_get_from_handles (priv->factory,
+		empathy_tp_contact_factory_get_from_handles (priv->connection,
 			added->len, (TpHandle *) added->data,
 			tp_chat_got_renamed_contacts_cb,
 			rename_data, (GDestroyNotify) contact_rename_data_free,
@@ -1141,7 +1137,7 @@ tp_chat_group_members_changed_cb (TpChannel     *self,
 
 	/* Request added contacts */
 	if (added->len > 0) {
-		empathy_tp_contact_factory_get_from_handles (priv->factory,
+		empathy_tp_contact_factory_get_from_handles (priv->connection,
 			added->len, (TpHandle *) added->data,
 			tp_chat_got_added_contacts_cb, NULL, NULL,
 			G_OBJECT (chat));
@@ -1155,7 +1151,7 @@ tp_chat_group_members_changed_cb (TpChannel     *self,
 }
 
 static void
-tp_chat_got_remote_contact_cb (EmpathyTpContactFactory *factory,
+tp_chat_got_remote_contact_cb (TpConnection            *connection,
 			       EmpathyContact          *contact,
 			       const GError            *error,
 			       gpointer                 user_data,
@@ -1172,11 +1168,11 @@ tp_chat_got_remote_contact_cb (EmpathyTpContactFactory *factory,
 	priv->remote_contact = g_object_ref (contact);
 	g_object_notify (chat, "remote-contact");
 
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
+	check_almost_ready (EMPATHY_TP_CHAT (chat));
 }
 
 static void
-tp_chat_got_self_contact_cb (EmpathyTpContactFactory *factory,
+tp_chat_got_self_contact_cb (TpConnection            *connection,
 			     EmpathyContact          *contact,
 			     const GError            *error,
 			     gpointer                 user_data,
@@ -1192,7 +1188,7 @@ tp_chat_got_self_contact_cb (EmpathyTpContactFactory *factory,
 
 	priv->user = g_object_ref (contact);
 	empathy_contact_set_is_user (priv->user, TRUE);
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (chat));
+	check_almost_ready (EMPATHY_TP_CHAT (chat));
 }
 
 static void
@@ -1230,7 +1226,7 @@ got_password_flags_cb (TpChannel *proxy,
 	priv->got_password_flags = TRUE;
 	priv->password_flags = password_flags;
 
-	tp_chat_check_if_ready (EMPATHY_TP_CHAT (self));
+	check_almost_ready (EMPATHY_TP_CHAT (self));
 }
 
 static GObject *
@@ -1240,15 +1236,13 @@ tp_chat_constructor (GType                  type,
 {
 	GObject           *chat;
 	EmpathyTpChatPriv *priv;
-	TpConnection      *connection;
 	TpHandle           handle;
 
 	chat = G_OBJECT_CLASS (empathy_tp_chat_parent_class)->constructor (type, n_props, props);
 
 	priv = GET_PRIV (chat);
 
-	connection = tp_channel_borrow_connection (priv->channel);
-	priv->factory = empathy_tp_contact_factory_dup_singleton (connection);
+	priv->connection = g_object_ref (tp_channel_borrow_connection (priv->channel));
 	g_signal_connect (priv->channel, "invalidated",
 			  G_CALLBACK (tp_chat_invalidated_cb),
 			  chat);
@@ -1260,14 +1254,14 @@ tp_chat_constructor (GType                  type,
 
 		/* Get self contact from the group's self handle */
 		handle = tp_channel_group_get_self_handle (priv->channel);
-		empathy_tp_contact_factory_get_from_handle (priv->factory,
+		empathy_tp_contact_factory_get_from_handle (priv->connection,
 			handle, tp_chat_got_self_contact_cb,
 			NULL, NULL, chat);
 
 		/* Get initial member contacts */
 		members = tp_channel_group_get_members (priv->channel);
 		handles = tp_intset_to_array (members);
-		empathy_tp_contact_factory_get_from_handles (priv->factory,
+		empathy_tp_contact_factory_get_from_handles (priv->connection,
 			handles->len, (TpHandle *) handles->data,
 			tp_chat_got_added_contacts_cb, NULL, NULL, chat);
 
@@ -1280,19 +1274,19 @@ tp_chat_constructor (GType                  type,
 		GList *list, *ptr;
 
 		/* Get the self contact from the connection's self handle */
-		handle = tp_connection_get_self_handle (connection);
-		empathy_tp_contact_factory_get_from_handle (priv->factory,
+		handle = tp_connection_get_self_handle (priv->connection);
+		empathy_tp_contact_factory_get_from_handle (priv->connection,
 			handle, tp_chat_got_self_contact_cb,
 			NULL, NULL, chat);
 
 		/* Get the remote contact */
 		handle = tp_channel_get_handle (priv->channel, NULL);
-		empathy_tp_contact_factory_get_from_handle (priv->factory,
+		empathy_tp_contact_factory_get_from_handle (priv->connection,
 			handle, tp_chat_got_remote_contact_cb,
 			NULL, NULL, chat);
 
 		list = empathy_dispatcher_find_requestable_channel_classes (
-			dispatcher, connection,
+			dispatcher, priv->connection,
 			tp_channel_get_channel_type (priv->channel),
 			TP_UNKNOWN_HANDLE_TYPE, NULL);
 
@@ -1497,7 +1491,6 @@ empathy_tp_chat_init (EmpathyTpChat *chat)
 		EMPATHY_TYPE_TP_CHAT, EmpathyTpChatPriv);
 
 	chat->priv = priv;
-	priv->contact_monitor = NULL;
 	priv->messages_queue = g_queue_new ();
 	priv->pending_messages_queue = g_queue_new ();
 }
@@ -1508,7 +1501,6 @@ tp_chat_iface_init (EmpathyContactListIface *iface)
 	iface->add         = tp_chat_add;
 	iface->remove      = tp_chat_remove;
 	iface->get_members = tp_chat_get_members;
-	iface->get_monitor = tp_chat_get_monitor;
 }
 
 EmpathyTpChat *
@@ -1862,3 +1854,53 @@ empathy_tp_chat_leave (EmpathyTpChat *self)
 
 	g_array_free (array, TRUE);
 }
+
+static void
+add_members_cb (TpChannel *proxy,
+		const GError *error,
+		gpointer user_data,
+		GObject *weak_object)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (weak_object);
+
+	if (error != NULL) {
+		DEBUG ("Failed to join chat (%s): %s",
+			tp_channel_get_identifier (priv->channel), error->message);
+	}
+}
+
+void
+empathy_tp_chat_join (EmpathyTpChat *self)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (self);
+	TpHandle self_handle;
+	GArray *members;
+
+	self_handle = tp_channel_group_get_self_handle (priv->channel);
+
+	members = g_array_sized_new (FALSE, FALSE, sizeof (TpHandle), 1);
+	g_array_append_val (members, self_handle);
+
+	tp_cli_channel_interface_group_call_add_members (priv->channel, -1, members,
+		"", add_members_cb, NULL, NULL, G_OBJECT (self));
+
+	g_array_free (members, TRUE);
+}
+
+gboolean
+empathy_tp_chat_is_invited (EmpathyTpChat *self,
+			    TpHandle *inviter)
+{
+	EmpathyTpChatPriv *priv = GET_PRIV (self);
+	TpHandle self_handle;
+
+	if (!tp_proxy_has_interface (priv->channel, TP_IFACE_CHANNEL_INTERFACE_GROUP))
+		return FALSE;
+
+	self_handle = tp_channel_group_get_self_handle (priv->channel);
+	if (self_handle == 0)
+		return FALSE;
+
+	return tp_channel_group_get_local_pending_info (priv->channel, self_handle,
+		inviter, NULL, NULL);
+}
